/*
 * MAX : laser.cc, Wed Jan  2 14:44:59 CET 2002 -sg
 *
 * Sample Device driver 
 * 
 * This file is part of Max data acquisition software
 * (c) 1998 by C. Rosen
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */



/*! \file laser.cpp
 *\brief Example devices file to demonstrate the use of the device-modules
 *
 *This file contains three sample devices :
 *
 *laser -> a device that does not take data.
 *adc   -> a device that has four channels of scalar integer.
 *scope -> a device that stores four channels of arrays of int.
 *
 *Each physical device in the experiment is represented by it's "device function" in the computer.
 *This function does all the communication with
 *your experiment.
 *For information, check also the files : devices.h, and priority
 *You can select the devices and their "channels"
 *from the main menu. This way the user can tell the server which devices
 * to call.
 */
/* **************************************************************
 *! \brief The user can use the  following routines to handle the device data
 *
 * IMPORTANT COMMANDS  :
 *
 * put variable  of type int from deviceparameter database
 *   int List::putvar(int *var,char *device_name,char *var_name)
 * get variable of type double  from deviceparameter database  :
 *   int List::getvar(double * var,char *device_name,char *var_name)
 * store measured data of different types :
 *  int Scheduler:.store(int var)
 *  int Scheduler:.store(int var[])
 *  int Scheduler:.store(double var)
 *  int Scheduler:.store(double var[])
 *
 * other useful commands :
 *  int Scheduler::channel_number(void)    -> get the actual channel number
 *  int Scheduler::act_integrations(void)  -> get the current integrations
 *  int Scheduler::max_integrations(void)  -> get the maximal integrations per channel
 *  int Scheduler::max_channels(void)      -> get the number of channels in this experiment
 *************************************************************/
/*************************************************************
 *   ERROR COMMANDS :
 * int Scheduler::error (char *formatstr,....)
 *   -> compose your individual errormessage
 * int Scheduler::error (char *ERRMSG,char *NAME)
 *   -> uses standard error messages from error.h
 * they return TRUE if the user wants to continue
 * they return FALSE in case of abort
 *  -> abort the measurement by returning FALSE from the device_function
 *************************************************************/
#include <stdio.h>
#include <math.h>
#include "list.hh"
#include "sched.hh"
#include "camacs.h"
#include "error.hh"

//some variables may be defined global for this file
//this way they don't have to reinitialized each time
int enabled=1;
int ind=0;
int result=0;
int a[500];
int c=100,f;
double b;
double phase;

//!this is the initfunction, it's called once before the measurement starts
int laser_init(List *device_data, Scheduler *parameters) {
	//printf("\"laser_init\" called\n");
  	int b= 0;
  	device_data->putvar(b,"laser","anfang");
	return 1;
}

//! Close your device after the measurement is finished
int laser_close(List *device_data, Scheduler *parameters){
  	//printf("\"laser_close\" called\n");
  	return 1;
}

//!this is the Pre_initfunction, it's called each time, before the integrastions cycle starts
int laser_pre(List *device_data, Scheduler *parameters){
  	//printf("\"laser_pre\" called\n");
  	return 1;
}

//! This is the main function of the  \b device \b module laser. Here the data is taken.
int laser(List *device_data, Scheduler *parameters) {
	double d;
  	//printf("\"laser\" called\n");
  	enabled = device_data->enabled("laser","kanal");
  	if (enabled) {
      		//printf("laser channel 1 enabled\n");
      		result = device_data->getvar(&b,"laser","phase");
      		result = device_data->getvar(&c,"laser","scale");
      		result = device_data->getvar(&d,"laser","steps");
      		b+=d;
      		device_data->putvar(b,"laser","phase");
      		result = device_data->getvar(&phase,"laser","phase");
      		for (result = 0;result <200;result++) {
	      		a[result] = (int) (c*sin( (b*result)/100.0));
      		}
      		parameters -> store(a,200);//10*(int) b);//*sin( b *0.01));
  	}
  	enabled = device_data->enabled("laser","sack");
  	if (enabled) {
    		//printf("laser channel 2 enabled\n");
    		int x = (int) (100.0*rand()/RAND_MAX);
		printf("x=%d\n",x);
    		parameters -> store(x);
  	}
  	return result; //zero aborts measurement
}

//!The initfunction is called once before the measurement starts
int adc_init(List *device_data, Scheduler *parameters) {
  printf("adc_init\n");
  return TRUE;	
}

//!this is the Pre_initfunction, it's called each time, before the integrastions cycle starts
int adc_pre(List *device_data, Scheduler *parameters) {
    printf("adc_pre\n");
    return TRUE;
}

//! This is the main function of the \b device \b module adc. Here the data is taken.
int adc(List *device_data, Scheduler *parameters) {
  double messwert;
  double gate;
  double gate2;
  int enabled=1;
  printf("\nadc called ");
  result = device_data->getvar(&gate,"adc","gate");
  enabled = device_data->enabled("adc","channel1"); //check if channel1 is enabled
  if(enabled) {
    printf("adc chan 1 is enabled \n");
    gate +=0.01;
    messwert = 1*sin(gate);
    result = device_data->putvar(gate,"adc","gate");
    parameters ->  store(messwert);
  }
  enabled = device_data->enabled("adc","channel2");  //check if channel is enabled
  if (enabled) {
    result = device_data -> getvar(&gate2,"adc","gate2");
    messwert = cos(gate2);
    gate2 += 0.01;
    result   = device_data -> putvar(gate2,"adc","gate2");
    parameters -> store(messwert);
  }
  enabled = device_data-> enabled("adc","channel3"); //check if channel3 is enabled
  if (enabled) {
    ind++;
    parameters -> store(ind);
  }
  return 1;
}

//! Close your device after the measurement is finished
int adc_close(List *device_data, Scheduler *parameters) {
  printf("adc close.................\n");
  return TRUE;
}

//!this is the initfunction, it's called once before the measurement starts
int scope_init(List *device_data, Scheduler *parameters) {
  printf("initializing the scope ...\n");
  return TRUE;
}

//!this is the Pre_initfunction, it's called each time, before the integrastions cycle starts
int scope_pre(List *device_data, Scheduler *parameters) {
  printf("presetting the scope......\n");
  return TRUE;
}

//! This is the main function of the \b device \b module scope. Here the data is taken.
int scope(List *device_data, Scheduler *parameters) {
  //check if first channel is enabled
  enabled = device_data->enabled("scope","chan_1");
  if (enabled) {
      printf("scope chan1 enabled");
      result     = device_data->getvar(&f,"scope","y_scale");
      printf("scope speed %d \n",f);
      result     = device_data->getvar(&c,"scope","speed");
      //
      //insert your device-readout routine here !!!!!
      //
      for (result = 0;result <200;result++) {
	a[result]=(int) (f*sin((result*c)/100));     //this function simulates measured data
      }
      parameters -> store(a,200);            //store measured data
  }
  //check second channel
  enabled = device_data->enabled("scope","chan_2");
  if (enabled) {
    result     = device_data->getvar(&f,"scope","y_scale");
    printf("scope channel 2 enabled\n");
    for (result = 0;result < 500; result++) {
      a[result] =(f*rand())/1000;
    }
    parameters -> store(a,500);
  }
  return TRUE;
}

//! Close your device after the measurement is finished
int scope_close(List *device_data, Scheduler *parameters) {
  printf("closing the scope \n");
  return TRUE;
}
