/*
 * MAX : le2323.cc, Fri Feb 15 18:11:21 CET 2002 -sg
 *
 * Device driver for the LeCroy 2323 Programmable Dual Gate Generator
 *
 * This file is part of Max data acquisition software
 * (c) 2002 S. Gerlach
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include "list.hh"
#include "sched.hh"
#include "error.hh"

extern "C" {
#include "camacs.h"
}

//#define DEBUG_DEV_LE2323

// global variables
static int  N, width_A, delay_A, start_delay_A, end_delay_A;
static int width_B, delay_B, start_delay_B, end_delay_B;

void set_word(int w, int d, int a) {
	int c=0, i=0, l=0; 
	unsigned int word=0;

	switch(w) {
	case 10:
		w=0;
		break;
	case 30:
		w=1;
		break;
	case 100:	
		w=2;
		break;
	case 300:
		w=3;
		break;
	default:
		printf("LE2323 : False Width! Setting to 100 ns\n");
		w=0;
		break;
	}

	if (d < 50 || d > 10.0e10)
		printf("LE2323 : False Delay\n");
	else if (d < 100)
		printf("LE2323 : reduced accurency and stability\n");

	while (i++ < 8) {
		if (d <= 1000)
			break;
		else {
			d /= 10;
			c +=1;
		}
	}

	// 
	c = c << 10;
	word = c | d;
	l = l << 13;
	word = l | word;
	w = w << 14;
	word = w | word;

	camac(17,a,N,&word);

#ifdef DEBUG_DEV_LE2323
	printf("Sending word %d a=%d\n",word,a);
	
	camac(1,0,N,&word);
	printf("Le2323 says : A-word = %d,",word);
	camac(1,0,N,&word);
	printf(" B-word = %d\n",word);
#endif
}

/*****************************************************************************/

//init function
//called each time before meausurement starts
int le2323_init(List *device_data, Scheduler *parameters) {	
  	if (camac_init() != 1)
    		if (!parameters->error(E_CAMAC,"Couldn't initialize device"))
      			return 0;

  	printf("LE2323 : Initializing\n");
  	device_data->getvar(&N,"le2323","N");
  	device_data->getvar(&width_A,"le2323","width_A");
  	device_data->getvar(&delay_A,"le2323","delay_A");
  	device_data->getvar(&start_delay_A,"le2323","start_delay_A");
  	device_data->getvar(&end_delay_A,"le2323","end_delay_A");
  	device_data->getvar(&width_B,"le2323","width_B");
  	device_data->getvar(&delay_B,"le2323","delay_B");
  	device_data->getvar(&start_delay_B,"le2323","start_delay_B");
  	device_data->getvar(&end_delay_B,"le2323","end_delay_B");

	// Setze width & delay
	set_word(width_A,start_delay_A,0);
	set_word(width_B,start_delay_B,1);

  return 1;
}

int le2323_pre ( List *device_data, Scheduler *parameters ) {
	int channel = parameters->act_channel();
	int chanmax = parameters->maximal_channels();

	if (chanmax >1) {
		delay_A = start_delay_A + 
			(channel-1)*(end_delay_A - start_delay_A)/(chanmax-1);
		delay_B = start_delay_B + 
			(channel-1)*(end_delay_B - start_delay_B)/(chanmax-1);
	}
	else {
		delay_A = start_delay_A;
		delay_B = start_delay_B;
	}

	// set delay
	if (start_delay_A != end_delay_A)
		set_word(width_A,delay_A,0);
	if (start_delay_B != end_delay_B)
		set_word(width_B,delay_B,1);

# ifdef DEBUG_DEV_LE2323
	printf("LE2323 : Setting Delay A to %d ns\n",delay_A);
	printf("LE2323 : Setting Delay B to %d ns\n",delay_B);
# endif

	device_data->putvar(delay_A,"le2323","delay_A");
	device_data->putvar(delay_B,"le2323","delay_B");

	return 1;
}

int le2323 ( List *device_data, Scheduler *parameters ) {
	return 1;
}

int le2323_close(List *device_data, Scheduler *parameters) {
	printf("LE2323 : Closing\n");
	
	device_data->putvar(delay_A,"le2323","delay_A");
	device_data->putvar(delay_B,"le2323","delay_B");
	
	if (camac_close() != 1)
		if (!parameters->error(E_CAMAC,"Couldn't close device"))
			return 0;

	return 1;
}
