 
#include "gpibfx.h"


/*
 sample gpib access routines for ines ieee488 driver
*/

//#define DEBUG_GPIBFX

int hdl=-1;


//----------------------------------------------------------------------------
int ieerror(char *msg)
//----------------------------------------------------------------------------
{
  printf("error: %s\n",msg);
  iedeinit();
  exit(0);
}


//----------------------------------------------------------------------------
int ieoutput(int pad,byte *buf, uint cnt, ulong tmo) {
//----------------------------------------------------------------------------	
	int ret;
	struct ie_devdep_cnfg  devCfg;
  	byte addrStr[]={
    	3,                   // len of command string
    	GPIB_TAG,            // board is talker
    	GPIB_UNL,            // unlisten 
    	GPIB_LAG             // listener (pad will be added)
  	};
  	
	addrStr[3]+=pad;       // add pad to GPIB_LAG

	
#ifdef DEBUG_GPIBFX
	printf("ieoutput()\n");
	printf("ioctl(IEIOC_RSDEVCFG)\n");
#endif
  	if(ioctl(hdl,IEIOC_RSDEVCFG,&devCfg)) { // read current config to get actual timeout
    		perror("ioctl(IEIOC_RSDEVCFG)");
		ieerror("enter: get dev config failed");
	}
 
  	devCfg.tmo_wr=tmo;
 
#ifdef DEBUG_GPIBFX
	printf("ioctl(IEIOC_WSDEVCFG)\n");
#endif
  	if(ioctl(hdl,IEIOC_WSDEVCFG,&devCfg)) { // set new timeout
		perror("ioctl(IEIOC_WSDEVCFG)");
    		ieerror("enter: set dev config failed");
	}
	
#ifdef DEBUG_GPIBFX
	printf("ioctl(IEIOC_BDCMDS)\n");
#endif
  	if(ioctl(hdl,IEIOC_BDCMDS,addrStr)) { // do the addressing 
		perror("ioctl(IEIOC_BDCMDS)");
    		ieerror("addressing for output failed");
	}
	
#ifdef DEBUG_GPIBFX
	printf("ieoutput()#write()\n");
#endif
  	ret = write(hdl,buf,cnt);         // write the data
  	
	if(ret != cnt) { 
    		printf("ret: %d  expected: %d\n",ret,cnt); 
    		//ieerror("write data failed");
  	}

  	return 0;
}


//----------------------------------------------------------------------------
int ieenter(int pad, byte *buf,uint cnt,ulong tmo) {
//----------------------------------------------------------------------------
  	int ret;
  	struct ie_devdep_cnfg  devCfg;

  	byte addrStr[]={
    	3,                 // len of command string 
    	GPIB_UNL,          // unlisten    
    	GPIB_LAG,          // board is listener
    	GPIB_TAG           // talker (pad will be added) 
  	};

  	addrStr[3]+=pad;     // add pad to GPIB_TAG
  
#ifdef DEBUG_GPIBFX
	printf("ieenter()\n");
	printf("ioctl(IEIOC_RSDEVCFG)\n");
#endif
	if(ioctl(hdl,IEIOC_RSDEVCFG,&devCfg)) { // read current config to get actual timeout
		perror("ioctl(IEIOC_RSDEVCFG)");
    		ieerror("enter: get dev config failed");
	}

#ifdef DEBUG_GPIBFX
	printf("ioctl(IEIOC_WSDEVCFG)\n");
#endif
	
  	devCfg.tmo_rd=tmo;

  	if(ioctl(hdl,IEIOC_WSDEVCFG,&devCfg)) { // set new timeout
		perror("ioctl(IEIOC_WSDEVCFG)");
    		ieerror("enter: set dev config failed");
	}

#ifdef DEBUG_GPIBFX
	printf("ioctl(IEIOC_BDCMDS)\n");
#endif
	
  	if(ioctl(hdl,IEIOC_BDCMDS,addrStr)) { // do the addressing 
		perror("ioctl(IEIOC_BDCMDS)");
    		ieerror("addressing for enter failed");
	}

#ifdef DEBUG_GPIBFX
	printf("ieenter#read()\n");
#endif
	
  	ret = read(hdl,buf,cnt);
  	
	if(ret<0) { 
		perror("read");
    		ieerror("read data failed");
  	}
  	else {
    		buf[ret] = 0; // terminate buffer
    		//printf("%d byte received:<%s>\n",ret,buf);
  	}
  	
  	return ret;
}


//----------------------------------------------------------------------------
int ieinit() {
//----------------------------------------------------------------------------
  byte initStr[3] = {2, 27, 'I'};
  byte ifcStr[3]  = {2, 27, 'i'};
  byte remStr[3]  = {2, 27, 'R'};                                                 

  if((hdl=open("/dev/ieee488/board0",O_RDWR)) < 0)
    ieerror("open board failed");
  
  if(ioctl(hdl,IEIOC_BDCMDS,initStr)) // init the board
   ieerror("init board failed");
  
  if(ioctl(hdl,IEIOC_BDCMDS,ifcStr)) // send ifc
   ieerror("send ifc failed");

  if(ioctl(hdl,IEIOC_BDCMDS,remStr)) // set remote enable
   ieerror("set remote enable failed");
  return 0;
}

//----------------------------------------------------------------------------
int iedeinit()
//----------------------------------------------------------------------------
{
  if(hdl>0) close(hdl);
  return 0;
}

//----------------------------------------------------------------------------
int iewaitSRQ(ulong tmo) {
//----------------------------------------------------------------------------
  struct ie_board_cnfg cfg;
  byte srqStr[3]  = {2, 27, 'W'};
  int t;
  time_t t1;
  if(ioctl(hdl,IEIOC_RBDCFG,&cfg)) // read current config to get actual timeout
    ieerror("wait srq: get board config failed");
  t=cfg.tmo_cmds;
  cfg.tmo_cmds=tmo;
  if(ioctl(hdl,IEIOC_WBDCFG,&cfg)) // set new timeout
    ieerror("wait srq: set board config failed");

  t1=time(NULL);

  if(ioctl(hdl,IEIOC_BDCMDS,srqStr)) {
    printf("time elapsed: %d\n",(int)(time(NULL)-t1));                    
    ieerror("cmd: waitSrq failed");
  }
  printf("time elapsed: %d\n",(int)(time(NULL)-t1));                    
  cfg.tmo_cmds=t;
  if(ioctl(hdl,IEIOC_WBDCFG,&cfg)) // reset to previous read timeout 
    ieerror("wait srq: set board config failed");
  printf("SRQ occured\n");
  return 0;
}


//----------------------------------------------------------------------------
int iespoll(int pad) {
//----------------------------------------------------------------------------
  byte rsp;
  int ret;

  byte spollStr1[]={
    4,                 // len of command string
    GPIB_UNL,          // unlisten
    GPIB_LAG,          // board is listener
    GPIB_TAG,          // talker (pad will be added)
    GPIB_SPE           // serial poll enable 
  }; 

  byte spollStr2[]={
    2,                 // len of command string
    GPIB_SPD,          // serial poll disable
    GPIB_UNT           // untalk 
  };

  spollStr1[3]+=pad;   // add pad to GPIB_TAG
  if(ioctl(hdl,IEIOC_BDCMDS,spollStr1)) // do the addressing
    ieerror("addressing for spoll failed");

  ret=read(hdl,&rsp,1);
  if(ret<0)
    ieerror("read data failed");
  printf("response:%d\n",rsp);

  if(ioctl(hdl,IEIOC_BDCMDS,spollStr2)) // do the unaddressing
    ieerror("unaddressing for spoll failed");
   
  return 0;
}
