/*
 * MAX : max.cc, 17.08.2002 -sg
 *
 * main file for  the client programm that contains the Qt-based GUI
 *
 * This file is part of Max data acquisition software
 * Copyright (C) 1997,98 Christian Rosen
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <vector>

#include <math.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>

#include "max.h"

//pixmap includes
#include "Pixmaps/filesave.xpm"
#include "Pixmaps/fileprint.xpm"
#include "Pixmaps/reload.xpm"
#include "Pixmaps/exit.xpm"
#include "Pixmaps/Jrecycle.xpm"
#include "Pixmaps/info_dot.xpm"

//#define DEBUG_MAX

#ifdef DEBUG_MAX
        #define INFO(text) printf(text)
#else
        #define INFO(text)
#endif

#define XSize 200
#define YSize 150

const char * dataSaveText = "Click this button to save all data currently in memory."
"You will be prompted for a file name.\n"
"You can also select the Save command from the File menu ";
const char * parameterPrintText = "Click this button to print the Device Parameters and settings of this measurement "
"You can also select the Print command from the File menu ";
const char * reloadText = "Click this button the reread the Main Initfile\n"
"The filename is given in the box in the bottom. You can edit it there\n"
"MAX will be reinitialized and all parameters will be set to the old values";
const char * exitText ="Click here to leave the programm";
const char * infoText ="Click here to see the Main documentation in the ~/doc directory";
const char * startText="Press here to start the measurement";
const char * pauseText="Press here to pause the Data Acquisition process"
" Press it again to continue";
const char * monitorText="In Monitor Mode data is continuously taken "
"and plotted into the appropriate plot windows.\n"
"Only the data of the last channel taken is stored in the lokal memory!\n "
"Hence you can only save the current visible dataset."
"Choose the Save Option in the Plot Window for this. "
"This mode is only for monitoring a signal while adjusting something, for example. "
"The channel numbers are not valid anymore, Stop the measurement with the STOP button.";

const char * triggerText="Internal Trigger : Measurement routine is triggered by internal timer. Only one Data Acquistion cycle (one channel) is performed, and then the Polling Deamon waits for the next timer event.\n"
"Set the timer delay in the timing options field of the MainMenu.\n"
"Don't use a device routine that waits for external trigger if you use internal trigger!\n"
"Select external trigger and everything runs as usual.";
const char * priorityText ="Priority Database : This listbox tells you which devices you are currently using and in which order they are called during the measurement.\n"
"Zero means that the device is not called at all.\n"
"Don't use identical numbers, and don't leave any numbers out in your priority list! \n";
const char * parameterText="Device Parameter Database : \n"
"This listbox contains all parameters of the devices currently used. \n"
"Double click on the parameter name to change the value.\n"
"The contents of this box are periodically updated. "
"To change the update intervall, see the update-option in the MenuBar."
"The STORE statement indicates that this is a channel of the device which contains measured data. "
"Each device can have different channels, which can be enabled via this listbox. "
"Double click on the Store statement, to toggle the plotstatus of this channel and to enable or disable it.";
const char * buttonText ="Enable the button also by pressing the ALT+character key";

void signalhandler( int signum ) {
	printf("MAX : Signal %d caught,\nPlease terminate the programm properly via the Menu \n",signum);
	printf("To make shure that Client and Server abort normally.\n");
	exit(1);
}

/**************************************************/
/*Plot buffer class definitions                   */
/*used to buffer plot data before plot_timer fires*/
/**************************************************/
PlotBuffer::PlotBuffer(int value,int channel,int index,int priority) {
	i_array = NULL;
	d_array = NULL;
	chan  = channel;
	indx  = index;
	prio  = priority;
	i_val = value;
	length= 1;
	type  = 1; //int
}

PlotBuffer::PlotBuffer(double value,int channel,int index,int priority) {
	i_array = NULL;
	d_array = NULL;
	chan = channel;
	indx = index;
	prio = priority;
	d_val= value;
	length= 1;
	type = 2;	 //double
}

PlotBuffer::PlotBuffer(int *a,int len,int channel,int index,int priority) {
	chan = channel;
	indx = index;
	prio = priority;
	length = len;
	type = 1; 	//int
	i_array = a; 	//just copy pointer
}

PlotBuffer::PlotBuffer(double *a,int len,int channel,int index,int priority) {
	chan = channel;
	indx = index;
	prio = priority;
	length = len;
	type = 2;		 //double
	d_array = a; 	//just copy the pointer
}

PlotBuffer::PlotBuffer(vector<int> *vec,int len,int channel,int index,int priority) {
	prio		= priority;
	chan		= channel;
	indx		= index;
	iVecPtr = vec;	//just copy the pointer
	type		= 1;				//int
	length	= len;
}

PlotBuffer::PlotBuffer(vector<double> *vec,int len,int channel,int index,int priority){
	prio		= priority;
	chan		= channel;
	indx		= index;
	dVecPtr = vec;	//just copy the pointer
	type		= 1;				//int
	length	= len;
}

/***************************************************/
/*                                                 */
/*   MAIN GUI CLASS DEFINITIONS START HERE         */
/*                                                 */
/***************************************************/
MainGUI::MainGUI(char *hostname,char *filename) :
	QMainWindow(0,"MAX Main Window") {
	max_init_filename = new char(80);
	ASSERT( max_init_filename != 0);
	strcpy(max_init_filename , filename);

	//Pixmaps for buttons in ToolBar below MenuBar
	QPixmap saveIcon, printIcon, reloadIcon,exitIcon, infoIcon;

	WindowList = new QList <Plot>;
	WindowList -> setAutoDelete(TRUE); //delete items when they are removed

	pr_filename = new char(80);
	ASSERT( pr_filename !=0 );
	init_filename = new char(80);
	ASSERT( init_filename != 0);	

	//reserve memory for all classes needed

	Schedule = new Database;
	assert(Schedule !=0);
	Devices = new Cl_List;
	assert(Devices !=0);
	Devices->initialize();

	//before doing anything else, check the network connection
	port     = DEFAULT_PORT;
	if (connectSockets(hostname) < 0) {
		printf("MAX : Error while trying to establish socket Connection to %s \n",hostname);
		printf("MAX : Bailing out\n");
		kill(pid,SIGKILL); //kill child (the Polld) if there is one
		::exit(-1);
	}

	/*
  *QSOCKETNOTIFERS :
  *create socket notifers to handle events on the sockets
  *connect received DevData Notifier with it's socket
  *Each time data is present at a socket, this notifer calls
  *a routine to read from the socket
  */
 RDataNotifier = new QSocketNotifier(devr_sfd,QSocketNotifier::Read,this);
 QObject::connect(RDataNotifier,SIGNAL(activated(int)),
		  this,SLOT(dataReceived()));
#ifdef DEBUG_MAX
	cout<<"MAX : Initialising QSocketNotifier\n";
#endif
 RCmdNotifier =  new QSocketNotifier(cmdr_sfd,QSocketNotifier::Read,this);
 QObject::connect(RCmdNotifier,SIGNAL(activated(int)),
		  this,SLOT(commandReceived()));
 DataNotifier = new QSocketNotifier(mdat_sfd,QSocketNotifier::Read,this);
 QObject::connect(DataNotifier,SIGNAL(activated(int)),
		  this,SLOT(parameterReceived()));
 DelayW           = new DelayWidget;
 connect(DelayW  ,SIGNAL(SIG_setdelay(uint ))
	 ,this, SLOT( apply_trigger_delay(uint)));

 lbtodata         = new int[10000];
 assert( lbtodata != 0);
 lbtoplotprops    = new int[ACTIVE_VARIABLES];
 assert ( lbtoplotprops != 0);

 //List to buffer variables that have to be plotted
 P_Buf = new QList<PlotBuffer>;
 P_Buf -> setAutoDelete(TRUE); //erase list from memory if clear() is called
 //init variable type buffer
 //this buffer is needed for save routine
 //gives order and types of data stored
 VarListPtr       = new VarType[ACTIVE_VARIABLES];
 assert(VarListPtr != 0);
 ArrayListPtr     = new ArrayType[ACTIVE_VARIABLES];
 assert(ArrayListPtr != 0);
 buf = new int[DATA_PACKET_SIZE];
 dbuf = new double[DATA_PACKET_SIZE];
 l_fname  = new char[100]; //filenames for name of last save names
 l_afname = new char[100];
 l_ifname = new char[100];
 memset(l_fname,0,sizeof(l_fname));
 memset(l_afname,0,sizeof(l_afname));
 memset(l_ifname,0,sizeof(l_ifname));
 m_running        = FALSE;
 trigger_period   = INT_TRIGGER_PERIOD;
 monitor_mode     = MONITOR_MODE;
 Schedule -> set_MonMode(monitor_mode);
 update_intervall = LISTB_UPDATE_INTERVALL;
 save_detect_enable = SAVE_DETECT; //If False save widget does not pop up automatically after measurement
 if ( update_intervall == 0)
     update_intervall = 2000;
 plot_update       = PLOT_WINDOW_UPDATE;
 if (plot_update == 0)
     plot_update = 500;
 color_state      = 0;
 dev_lb_update    = TRUE;
 save_flag        = TRUE; //start with empty database
 dev_index =0; //these two are use to toggle plot/enabled flags in the databases
 device_lb_flag = FALSE; //listbox is empty at startup
 plot_index = 0;
 max_channels     = 1;  //initialize to default values
 max_integrations = 1;
 iarraybuffer = new int[MAX_ARRAY_SIZE];
 assert(iarraybuffer != 0);
 darraybuffer = new double[MAX_ARRAY_SIZE];
 assert(darraybuffer != 0);
 
 // Build up GUI-frontend and connect all buttons ...
 // set red color group
 col    = QColor( 240,10, 10);
 col_off= QColor( 100,20,10 );
 col_on = QColor( 230,20,30 );
 colgrp = QColorGroup(black, col, col.light(),
		      col.dark(),col.dark(120),cyan, white);
 colgrp_off = QColorGroup(black, lightGray,
			  white, black,col_off.dark(120),red, white);
 colgrp_on  = QColorGroup(black, lightGray,
			  white, black,col_off.dark(120),black, white);
 //white background when data to be entered
 grp_active = QColorGroup(black,white,white,black,
			  black,black,white);
 //dark backgroundcolor if passive
 grp_passive = QColorGroup(black,lightGray,lightGray,lightGray,
			   black,black,lightGray);
 //fist set some variables and sizes
 setMinimumSize(MAIN_GUI_X_SIZE,MAIN_GUI_Y_SIZE);
 
 //Set up toolbar with icons
 fileTools = new QToolBar( this, "File Operations" );
 saveIcon = QPixmap( filesave );
 QToolButton * fileSave = new QToolButton( saveIcon, "Save Data", 0,
					      this, SLOT(save()),
					      fileTools, "save data" );
 printIcon = QPixmap( fileprint );
 QToolButton * filePrint = new QToolButton( printIcon, "Print Parameters", 0,
					    this, SLOT(print()),
					    fileTools, "print parameters" );
 reloadIcon = QPixmap( rec_xpm );
 QToolButton * reload = new QToolButton( reloadIcon, "Reload Main Initfile", 0,
					    this, SLOT(reinitAll()),
					    fileTools, "reload main initfile" );
 exitIcon = QPixmap( not_marked_xpm);
 QToolButton * exitMax = new QToolButton( exitIcon, "Exit MAX", 0,
					    this, SLOT(bye()),
					    fileTools, "Exit MAX" );
 infoIcon = QPixmap( info_dot );
 QToolButton * info = new QToolButton( infoIcon, "Help_infos", 0,
					    this, SLOT(help()),
					    fileTools, "Help Infos" );
 (void)QWhatsThis::whatsThisButton( fileTools );  //add help button

#ifdef _QT2_
 QWhatsThis::add( fileSave,  dataSaveText);
 QWhatsThis::add( filePrint, parameterPrintText);
 QWhatsThis::add( reload,    reloadText);
 QWhatsThis::add( exitMax,   exitText);
 QWhatsThis::add( info,      infoText);
#else
 QWhatsThis::add( fileSave,  dataSaveText, FALSE );
 QWhatsThis::add( filePrint, parameterPrintText, FALSE );
 QWhatsThis::add( reload,    reloadText, FALSE );
 QWhatsThis::add( exitMax,   exitText, FALSE );
 QWhatsThis::add( info,      infoText, FALSE );
#endif
 
 //create Menu bar and popup Menues
 QPopupMenu *file = new QPopupMenu;
 CHECK_PTR( file );
 file->insertItem( "Open" , this, SLOT(open ()), CTRL+Key_O );
 file->insertItem( "Save" , this, SLOT(save ()), CTRL+Key_S );
 file->insertItem( "Print", this, SLOT(print()));
 file->insertSeparator();
 file->insertItem( "Exit",  this ,SLOT(bye  ()), CTRL+Key_Q );
 file->insertSeparator();
 QPopupMenu *help = new QPopupMenu;
 CHECK_PTR( help );
 help ->insertItem( "About MAX", this, SLOT(about()), CTRL+Key_A );
 help ->insertItem( "Help Topic", this, SLOT(help()), CTRL+Key_H );

 QPopupMenu *init = new QPopupMenu;
 CHECK_PTR( init );
 init ->insertItem( "Reread Device and Priority Initfiles",
		    this, SLOT(reinit()), CTRL+Key_D);
 init -> insertItem( "Reread Priority file only ",
		    this, SLOT(reinit_priority()));
 init -> insertItem("Browse for Priority-filename",
		    this,SLOT( browsePrio()));
 init -> insertItem( "Reread Device file only ",
		    this, SLOT(reinit_devices()));
 init -> insertItem("Browse for Device-filename",
		    this,SLOT( browseDev()));
 init -> insertItem( "Reread Main Initfile ",
		    this,SLOT (reinitAll()),CTRL+Key_R);
 init -> insertItem ("Browse for Main-initfilname",
		     this,SLOT( browseMain() ) );
 QPopupMenu *update = new QPopupMenu;
 CHECK_PTR( update );
 update -> insertItem( "Update Device Listbox",
		     this,SLOT(listbox_update()),CTRL+Key_U);
 update -> insertItem( "Update Priority Listbox",
		     this, SLOT(priority_update()));
 update -> insertItem("Set Update Timing",
		      this,SLOT(set_plotwindow_update_intervall()));
 QPopupMenu *Trigger = new QPopupMenu;
 CHECK_PTR( Trigger );
 Trigger->insertItem( "internal Trigger",this,SLOT(set_trigger_internal()));
 Trigger->insertItem( "external Trigger",this,SLOT(set_trigger_external()));
 Trigger->insertItem( "Set Trigger Delay",this,SLOT(set_trigger_delay()));

 QPopupMenu *Options = new QPopupMenu; // options for miscelaneous stuff
 CHECK_PTR( Options );
 Options -> insertItem("Allways show Save-Widget",this,SLOT(toggle_save_widget_enable()));
 Options -> insertItem("Never show Save-Widget",this,SLOT(toggle_save_widget_disable()));

 menu = new QMenuBar( this );
 CHECK_PTR( menu );
 menu -> insertItem( "&File",   file    );
 menu -> insertItem( "&Init",   init    );
 menu -> insertItem( "&Update", update  );
 menu -> insertItem( "&Trigger",Trigger );
 menu -> insertItem( "&Options",Options );
 //  menu ->insertItem( "Plot_Options",SelectPlot);
 menu ->insertSeparator();
 menu ->insertItem( "&Help", help );
 
 	// create the top layout
 	// this is a vertical box to arrange items to the bottom
#ifdef _QT2_
 	QBoxLayout *topLayout = new QVBoxLayout(layout()); 
#else
 	QBoxLayout *topLayout = new QVBoxLayout( this, 2);
#endif
 
 topLayout -> setMenuBar( menu );
 topLayout -> addWidget( fileTools);
 //
 //the second row contains a few buttons, the channel number widget
 //and the QlineEdits for Integrations and channels
 //
 QHBoxLayout *Headline = new QHBoxLayout(10);
 topLayout-> addLayout ( Headline );

 int headx=5,heady=3;   	 //headline is a 5*2 grid
 QGridLayout *HeadGrid = new QGridLayout(heady,headx);
	Headline -> addLayout ( HeadGrid );

 //set labels for Trigger mode , speed and to show if measurement is running

 button_triggermode = new QPushButton( this, "button_triggermode");
 if (trigger_mode)
   button_triggermode -> setText("Trigger Mode : EXTERNAL");
 else
   button_triggermode -> setText("Trigger Mode : INTERNAL");
 button_triggermode -> setFont(QFont("Helvetica",14,QFont::Bold));
 connect(button_triggermode,SIGNAL(clicked()),SLOT(toggle_trigger_mode()));
#ifdef _QT2_ 
	QWhatsThis::add( button_triggermode,   triggerText);
#else 
 	QWhatsThis::add( button_triggermode,   triggerText, FALSE );
#endif
 
 button_monitormode = new QPushButton( this, "button_monitormode"  );
 if (!monitor_mode)
   button_monitormode -> setText("Data Aquisition Mode");
 else
   button_monitormode -> setText("    Monitor Mode    ");
 button_monitormode -> setFont(QFont("Helvetica",14,QFont::Bold));
 connect(button_monitormode,SIGNAL(clicked()),SLOT(toggle_monitor_mode()));
#ifdef _QT2_ 
 QWhatsThis::add( button_monitormode,   monitorText);
#else
 QWhatsThis::add( button_monitormode,   monitorText, FALSE ); 	   // add text to buttons
#endif 
 //label for channel number in the middle of the main window

 chan_no = new QLCDNumber(6, this, "channelnumber");
 chan_no -> setSegmentStyle(QLCDNumber::Flat);
 chan_no -> setFrameStyle( QFrame::Panel | QFrame::Plain );
 chan_no -> setLineWidth( 2 );
 //  chan_no -> setSegmentStyle(SegmentStyleQLCDNumber::Filled);

 chn_label = new QLabel(this, "chn_label" );
 chn_label -> setText("Stop");
 chn_label -> setFont(QFont("Helvetica",18,QFont::Bold));
 chn_label -> setAlignment( AlignCenter | AlignVCenter );
 chn_label -> setAutoResize( FALSE);

 //set text label for max channel numbers
 label_max_channels = new QLabel( this, "label_max_channels");
 label_max_channels -> setText("      Channels  :");
 label_max_channels -> setFont(QFont("Helvetica",12,QFont::Bold));
 label_max_channels -> setAlignment(AlignCenter);
 label_max_channels -> setAutoResize( TRUE );

 //set editable box to enter maximal channel number
 channel_editor = new QLineEdit(this, "channel_editor");
 channel_editor -> setMaxLength(10);
 channel_editor -> setFixedHeight(channel_editor -> sizeHint().height());
 channel_editor -> setFont(QFont("Helvetica",12,QFont::Bold));
#ifdef _QT2_
 connect(channel_editor,SIGNAL(textChanged(const QString &)),SLOT(set_maxnum()));
#else
 connect(channel_editor,SIGNAL(textChanged(const char*)),SLOT(set_maxnum()));
#endif 

 //set text label for integrations
 label_integrations = new QLabel( this, "label_max_channels");
 label_integrations -> setText(" Integrations   :");
 label_integrations -> setFont(QFont("Helvetica",12,QFont::Bold));
 label_integrations -> setAlignment(AlignCenter);
 label_integrations -> setAutoResize( TRUE );

 //set editable box to enter integrations
 integ_editor = new QLineEdit(this,"integrations_editor");
 integ_editor -> setFont(QFont("Helvetica",12,QFont::Bold));
 integ_editor -> setMaxLength(10);
 integ_editor -> setFixedHeight(integ_editor -> sizeHint().height());
#ifdef _QT2_
 connect(integ_editor,SIGNAL(textChanged(const QString &)),SLOT(set_integrations()));
#else
 connect(integ_editor,SIGNAL(textChanged(const char*)),SLOT(set_integrations()));
#endif 

 //add all headline widgets to grid layout
 HeadGrid -> addMultiCellWidget( button_triggermode, 0,0,0,1 );
 HeadGrid -> addMultiCellWidget( button_monitormode, 2,2,0,1 );
 HeadGrid -> addMultiCellWidget( chan_no,0,1,2,2);
 HeadGrid -> addWidget( chn_label,2,2);
 HeadGrid -> addWidget(label_max_channels,0,3,AlignVCenter);
 HeadGrid -> addWidget(label_integrations,2,3,AlignVCenter);
 HeadGrid -> addWidget(channel_editor,0,4);
 HeadGrid -> addWidget(integ_editor,2,4);

 //insert top separator to seperate head from main part

 top_separator = new QFrame( this ," ");
 top_separator -> setFrameStyle( QFrame::HLine | QFrame::Sunken );
 topLayout -> addWidget(top_separator);

 //this is the middle part containg the start/stop buttons
 //and the listboxes
 //QBoxLayout *MainPart = new QHBoxLayout();

 int gridx = 3;
 int gridy = 6;
 QGridLayout *MainGrid = new QGridLayout(gridy,gridx);
 topLayout-> addLayout ( MainGrid);
 
 //create Pushbuttons for Start,Stop,Cont and Reset

 pbstart= new QPushButton ("ST&ART",this);
 pbstart -> setFont( QFont("Arial",12,QFont::Bold));
 pbstart -> setMinimumSize(MIN_BUTTON_SIZE,MIN_BUTTON_SIZE );
 connect(pbstart,SIGNAL(clicked()),SLOT(startclicked()) );
 QToolTip::add(pbstart, "Start Measurement");
#ifdef _QT2_ 
 	QWhatsThis::add( pbstart,   startText);
#else
 	QWhatsThis::add( pbstart,   startText, FALSE );
#endif	
 pbstop = new QPushButton ("&STOP", this);
 pbstop -> setFont( QFont("Arial",12,QFont::Bold));
 pbstop -> setPalette( QPalette(colgrp,colgrp,colgrp) );
 pbstop -> setMinimumSize(MIN_BUTTON_SIZE,MIN_BUTTON_SIZE );
 connect(pbstop,SIGNAL(clicked()),SLOT(stopclicked()) );
 QToolTip::add(pbstop, "Stop Measurement");

 pbpause = new QPushButton ("&PAUSE",this);
 pbpause ->setFont( QFont("Arial",12,QFont::Bold));
 pbpause -> setMinimumSize(MIN_BUTTON_SIZE,MIN_BUTTON_SIZE );
 connect(pbpause,SIGNAL(clicked()),SLOT(pauseclicked()) );
#ifdef _QT2_ 
 	QWhatsThis::add( pbpause,   pauseText);
#else
 	QWhatsThis::add( pbpause,   pauseText, FALSE );
#endif
 pbpause ->setPalette( QPalette(colgrp_off,colgrp_off,colgrp_off) );
 pbcont = new QPushButton ("&RESUME",this);
 pbcont ->setFont( QFont("Arial",12,QFont::Bold));
 pbcont -> setMinimumSize(MIN_BUTTON_SIZE,MIN_BUTTON_SIZE );
 QToolTip::add(pbcont, "Resume stopped Measurement");
 connect(pbcont,SIGNAL(clicked()),SLOT(contclicked()) );

 pbreset = new QPushButton ("&RESET",this);
 pbreset ->setFont( QFont("Arial",12,QFont::Bold));
 pbreset -> setMinimumSize(MIN_BUTTON_SIZE,MIN_BUTTON_SIZE );
 connect(pbreset,SIGNAL(clicked()),SLOT(resetclicked()) );
 pbreset ->setPalette( QPalette(colgrp,colgrp,colgrp) ); //give red color
 QToolTip::add(pbreset, "Resetting");

#ifdef _QT2_
 QWhatsThis::add( pbstart,  buttonText);
 QWhatsThis::add( pbstop,   buttonText);
 QWhatsThis::add( pbpause,  buttonText);
 QWhatsThis::add( pbcont,   buttonText);
#else
 QWhatsThis::add( pbstart,  buttonText, FALSE );
 QWhatsThis::add( pbstop,   buttonText, FALSE );
 QWhatsThis::add( pbpause,  buttonText, FALSE );
 QWhatsThis::add( pbcont,   buttonText, FALSE );
#endif
 
 //create listbox for device data
 //and line edit to edit device parameters
 device_lb  = new QListBox(this, "Device_data");
 connect(device_lb,SIGNAL(selected( int )), SLOT(edit_parameter(int ) ) );
#ifdef _QT2_
 QWhatsThis::add( device_lb,   parameterText);
#else
 QWhatsThis::add( device_lb,   parameterText, FALSE );
#endif 
 // QToolTip::add(device_lb,"Click on variable to edit");
 dev_editor = new QLineEdit(this, "deveditor");
 dev_editor -> setMaxLength(30); //maximum length of characters
 dev_editor -> setFixedHeight(dev_editor -> sizeHint().height());
 dev_editor -> setPalette(QPalette (grp_passive,grp_passive,grp_passive));
 connect(dev_editor,SIGNAL(returnPressed( )), SLOT(set_new_parameter()));
 //set up priority listbox and editor field
 //
 prio_lv =new QListView(this, "priority");
 prio_lv ->addColumn( "Device Name" );
 prio_lv ->addColumn( "Priority" );
 connect(prio_lv,SIGNAL(doubleClicked( QListViewItem * )),SLOT(edit_priority(QListViewItem *) ) );
 connect(prio_lv,SIGNAL(rightButtonPressed( QListViewItem *,const QPoint &, int)),
	 SLOT(prio_lv_PopUp( QListViewItem *,const QPoint &, int)));
#ifdef _QT2_ 
 QWhatsThis::add( prio_lv,   priorityText);
#else
 QWhatsThis::add( prio_lv,   priorityText, FALSE );
#endif
 
 //PopUpMenu for switching device on/off
 priority_trigger = new QPopupMenu;
 priority_trigger -> insertItem("Enable",
				this,SLOT(Prio_Enabled()));
 priority_trigger -> insertItem("Disable",
				this,SLOT(Prio_Disabled()));
 priority_trigger -> setCheckable(TRUE);
 prio_editor = new QLineEdit(this,"Prio_editor");
 prio_editor -> setMaxLength(5); //-> not more that 99999 devices
 prio_editor -> setFixedHeight(prio_editor -> sizeHint().height());
 prio_editor -> setPalette(QPalette (grp_passive,grp_passive,grp_passive));
 connect(prio_editor,SIGNAL(returnPressed()),SLOT(set_priority()));

 MainGrid -> addWidget ( pbstart, 0,0 );
 MainGrid -> addWidget ( pbstop,  1,0 );
 MainGrid -> addWidget ( pbpause, 2,0 );
 MainGrid -> addWidget ( pbcont,  3,0 );
 MainGrid -> addWidget ( pbreset, 4,0 );
 MainGrid -> addMultiCellWidget(device_lb,0,4,1,1);
 MainGrid -> addMultiCellWidget(prio_lv,0,4,2,2);
 MainGrid -> addMultiCellWidget(dev_editor,5,5,1,1);
 MainGrid -> addMultiCellWidget(prio_editor,5,5,2,2);
 MainGrid -> setColStretch(1,20);
 MainGrid -> setColStretch(2,10);
 //insert separator line
 bottom_separator = new QFrame( this ," ");
 bottom_separator -> setFrameStyle( QFrame::HLine | QFrame::Sunken );
 topLayout -> addWidget(bottom_separator);
 //last line contains all file infos
 QBoxLayout *InfoLine   = new QHBoxLayout();
 QBoxLayout *InfoLabels = new QHBoxLayout();
 topLayout -> addLayout( InfoLabels );
 topLayout -> addLayout( InfoLine );

 prio_label = new QLabel ( this, "prio_fname");
 prio_label -> setText("Priority file");
 prio_label -> setAlignment(AlignCenter);
 prio_fname =  new QLineEdit(this,"priority_editor");
 // prio_fname -> setFont(QFont("Helvetica",12));
 prio_fname -> setMaxLength(100);
 prio_fname -> setPalette(QPalette(grp_passive,grp_passive,grp_passive) );
 // prio_fname -> setFixedHeight(prio_fname -> sizeHint().height());
 connect(prio_fname,SIGNAL(returnPressed()),SLOT(edit_pfile()));

 dev_label = new QLabel ( this, "prio_fname");
 dev_label -> setText("Device-Parameter initfile");
 dev_label -> setAlignment(AlignCenter);

 dev_fname =  new QLineEdit(this,"dev_fname_editor");
 dev_fname -> setMaxLength(100);
 dev_fname -> setPalette(QPalette(grp_passive,grp_passive,grp_passive) );
 connect(dev_fname,SIGNAL(returnPressed()),SLOT(edit_dfile()));

 init_label = new QLabel ( this, "prio_fname");
 init_label -> setText("MAX Initfile");
 init_label -> setAlignment(AlignCenter);
 in_fname =  new QLineEdit(this,"init_fname_editor");
 in_fname -> setMaxLength(100);
 connect(in_fname,SIGNAL(returnPressed()),SLOT(edit_ifile()));
 in_fname -> setPalette(QPalette(grp_passive,grp_passive,grp_passive) );

 //add all widgets to layout
 InfoLabels -> addWidget(prio_label,AlignVCenter | AlignHCenter );
 InfoLabels -> addWidget(dev_label, AlignVCenter | AlignHCenter );
 InfoLabels -> addWidget(init_label,AlignVCenter | AlignHCenter );
 InfoLine   -> addWidget(prio_fname);
 InfoLine   -> addWidget(dev_fname);
 InfoLine   -> addWidget(in_fname);

 progressTitle = new QLabel(this, "progressTitle");
 progressTitle -> setText("Progress of Measurement .....");
 progressTitle -> setAlignment( AlignCenter );

 progress = new QProgressBar(this,"mem-usage");
 topLayout -> addWidget( progressTitle );
 topLayout -> addWidget( progress );

 int_progress = new QProgressBar(this,"integ_progress");
 topLayout -> addWidget( int_progress );

 //set up timers for button blinking
 blink_timer = new QTimer(this);
 connect(blink_timer,SIGNAL(timeout()),
	  this, SLOT(ButtonTimerEvent()) );
 //timer for update of channel number widget
 number_timer = new QTimer(this);
 connect(number_timer,SIGNAL(timeout()),
	  this, SLOT(RefreshNumber()) );
 //timer to empty Plotbuffer Queue
 update_timer = new QTimer(this);
 connect(update_timer,SIGNAL(timeout()),
	 this, SLOT(UpdatePlotWindows()) );
 // Install an application-global event filter for
 // handling color change of QLineEdits
 qApp->installEventFilter( this );

 number_timer -> start( update_intervall, TRUE );
 
 //
 //now fill the databases....................
 //
#ifdef DEBUG_MAX
 	printf("MAX : Reading config files .... \n");
#endif
 
	if (reinitAll() == 0) {
   		if (!sendCommand(KILL_YOURSELF,"dummy",0)) {
     			printf("ERROR : MAX could not send kill command\n");
     			printf("Application will be terminated\n");
   		}	
   		kill(pid,-9);
   		exit(-1);//QApplication::exit();
 	}
 	sendAllDeviceData();
 	sendAllPriorities();
 	if (!sendCommand(PRIORITY_CHECK,"dummy",0)) {
		printf("ERROR : MAX could not send PRIORITY_CHECK command \n");
 	}	
 	in_fname   -> setText(max_init_filename);
 	prio_fname -> setText(pr_filename);
 	dev_fname  -> setText(init_filename);
 	W = new UpdateWidget(0);
 	connect(W  ,SIGNAL(SIG_setupdate(uint ,uint))
	 	,this, SLOT( apply_update_intervall(uint,uint)));
 	data_send_intervall = DATA_SEND_INTERVALL;
 	
	//now establish the signal handler
 	this -> repaint();
}

MainGUI::~MainGUI() {
}

// SLOTS called from Menu Bar PopUpMenus
void MainGUI::open() {
}

//Show the save widget after measurement is finished or
//the user has pressed the save button
void MainGUI::save() {
  string parameterStr;
  char *str = new char[200];
  QString string;
  QString buffer;
  char *_ifname,*_afname,*_fname;
  _ifname = new char[100];
  _afname = new char[100];
  _fname  = new char[100];
  memset(_ifname,0,sizeof(_ifname));
  memset(_afname,0,sizeof(_afname));
  memset(_fname,0,sizeof(_fname));
  date = QDate :: currentDate(); // get actual date
  time = QTime :: currentTime(); //set time
  sprintf(_fname,"%d_%d%s",date.month(),date.day(),save_fname);
  sprintf(_afname,"%d_%d%s",date.month(),date.day(),array_fname);
  sprintf(_ifname,"%d_%d%s",date.month(),date.day(),image_fname);

  //this tabDialog has be newly created end deleted each time you use it
  //it's it's own parent widget
  SaveW = new SaveWidget;
  connect(SaveW  ,SIGNAL(SIG_save(const char *,const char *,
				  const char *,int ,char *,
				  const char *,const char *)) ,
	  this, SLOT(do_save(const char *,const char *,
			     const char *,int ,char *,
			     const char *,const char *)));
  SaveW-> SetFilename(_fname,_afname,_ifname,data_dir,l_fname,l_afname,l_ifname);
  strncpy(l_fname,save_fname,sizeof(save_fname));
  strncpy(l_afname,array_fname,sizeof(array_fname));
  strncpy(l_ifname,image_fname,sizeof(image_fname));
  //construct string to plot in multiline edit
  string.sprintf("MAX, ver 2.3 by S.Gerlach\n  Copyright MBI, 2002 \n\n");
  buffer.sprintf("Measurement of    : %s, %d.%d.%d\n",(const char *) date.dayName(date.dayOfWeek()),
		 date.day(),date.month(),date.year());
  string +=buffer;
  buffer.sprintf("Time              : %d:%d:%d\n",time.hour(),time.minute(),
		 time.second() );
  string += buffer;
  buffer.sprintf("Channels          : %d\n",max_channels);
  string += buffer;
  buffer.sprintf("Integrations      : %d\n",max_integrations);
  string += buffer;
  string += "Device Parameters : \n";
  int pr =1;
   while ((str = Priority.Get_dev_name(pr))) {
     parameterStr = Devices->getAllDeviceParameters(str);
     string += parameterStr.c_str();
    pr+=1;
   }
   SaveW-> SetText(string);
}

void MainGUI::do_save(const char *fname,const char *arrayfname,
		      const char *imagefilename,int index,char *format,
		      const char *text, const char* new_dir) {
  strncpy(data_dir,new_dir,strlen(new_dir));
  double numberstores=0;
  char* str;
  char *strs;
  QString fname2;
  char * parameterStr;
  strcpy(l_fname,fname);
  parameterStr = new char[10000];//reserve space for buffer string
  str  = new char[100];           //buffer string for filename
  strs = new char[100];
  //length of device dataset is therere limited to 5000 signs ->
  //increase this if neccessary
  if (Schedule -> database_empty()) //==NULL -> database empty
    fname2 = fname;        //copy filename into buffer
  else
    if (Schedule -> array_empty())
      fname2=arrayfname;
    else
      fname2=imagefilename;
  strcpy(str,fname);    	 //append filename
  strcpy(strs,arrayfname);    	//append filename for integer arrays
  strcat (strs,".str"); 	//filename for arrays
  strcat (str, ".dat"); 	//append .dat extender
  QApplication::setOverrideCursor( waitCursor ); //cursor looks like wristwatch
  PlotProps 	*X; //Count number of store statements of all enabled devices with prio != 0
  for (X=Devices->PlotPropList -> first(); X != NULL;X = Devices->PlotPropList -> next())
    if (X -> priority != 0 && X -> storeflag != 0)
      numberstores ++;
   printf("MAX : numberstores %f lenght of array %d lenght of int %d\n",numberstores,
		   Schedule-> length_of_arraylist(),Schedule-> length_of_list());
  Schedule -> save_data((const char*) str,VarListPtr,varcounter);
  Schedule -> save_arrays((const char *) strs,ArrayListPtr); //save the arrays in an extra format
  Schedule -> save_images(imagefilename, index, format);
  save_flag = TRUE;
  QApplication::restoreOverrideCursor();
  //now save the parameter sets
  //loop over priorities to check active status of each device
  FILE *fp;	
  fname2 += ".txt";
  if ((fp=fopen(fname2,"w+"))==NULL) {
    int result=Ierror(E_OPEN,"for saving info-file","");
    result = 0; //used to get rid of compiler warning
    return;
  }
  fprintf(fp,"%s",text);
  fclose(fp);
  delete str;
  delete strs;
  delete parameterStr;
}

void MainGUI::print(){
}

void MainGUI::bye() {
	if (!m_running) {
    		bool ok = QMessageBox::query("ATTENTION!!",
			"Do You really want to quit this application ?","YES","NO");
		if (ok) {
			if (!sendCommand(KILL_YOURSELF,"dummy",0)) {
	    			printf("ERROR : MAX could not send kill command \n");
	    			return;
			}	
#ifdef DEBUG_MAX
			printf("MAX : Shutting down sockets ... ");
#endif			
			int a1 = shutdown(data_sfd,2);
			int a2 = shutdown(devr_sfd,2);
			int a3 = shutdown(cmd_sfd,2);
			int a4 = shutdown(cmdr_sfd,2);
			int a5 = shutdown(mdat_sfd,2);
#ifdef DEBUG_MAX
			printf("%d %d %d %d %d OK\n",a1,a2,a3,a4,a5);
#endif			

			// This causes segmentation violations !!!			
			//close(data_sfd);
			//close(devr_sfd);
			//close(cmd_sfd);
			//close(cmdr_sfd);
 			//close(mdat_sfd);
#ifdef DEBUG_MAX
			printf("MAX : Client has closed all his sockets \n");
#endif

			QApplication::exit();
		}
	}
}

void MainGUI::browseMain() { 
	// start at the current working directory and with *.cpp as filter
	QString f = QFileDialog::getOpenFileName( 0, "*", this , 
						"Main Initfile" );
 	if ( !f.isEmpty() ) {
 		strcpy(max_init_filename,f);// the user selected a valid existing file
		in_fname -> setText(max_init_filename);
 	} 
}

void MainGUI::browsePrio() {
    // start at the current working directory and with *.cpp as filter
    QString f = QFileDialog::getOpenFileName( 0, "*", this );
    if ( !f.isEmpty() ) {
       strcpy(pr_filename,f); // the user selected a valid existing file
       prio_fname -> setText(pr_filename);
    } else {
        // the user cancelled the dialog
    }
}
void MainGUI::browseDev() { 
	// start at the current working directory and with *.cpp as filter
    QString f = QFileDialog::getOpenFileName( 0, "*", this ,"Device-Parameters");
    if ( !f.isEmpty() ) {      // the user selected a valid existing file
      strcpy(init_filename,f); //copy filename to deviceParameters initfilename
      dev_fname -> setText(init_filename);
    } else {
        // the user cancelled the dialog
    }
}

void MainGUI::about() {
	QMessageBox::information(this,"MAX version 2.3 release 3 10/02","Copyright 2002,\n
		Max-Born Institute Berlin,\n       Division B2 \n
		e-mail :  gerlach@mbi-berlin.de\n");
}

/*
 * reinitialization routines
 * always send a  clear database command to the polld
 * before sending the new data
 * otherwise the polld just modifies the existing database
 */
void MainGUI::reinit() {
	device_lb_flag = FALSE; //device listbox will be cleared and refilled
	if (Priority.Pr_initlist(pr_filename)==0) {
		printf("I will quit the applications \n");
    		/* getchar();
    		if (!sendCommand(KILL_YOURSELF,"dummy",0)) {
      			printf("ERROR : Max could not send kill command \n");
      			return;
    		}	
    		QApplication::exit(); */
    	}
  int result = Devices->initlist(init_filename);
  if (result== -1) {
    	if (!sendCommand(KILL_YOURSELF,"dummy",0)) {
	    printf("ERROR : MAX could not send kill command \n");
	    return;
	}	
	QApplication::exit();
  }
  listbox_update();
  priority_update();
  if  (!sendCommand(CLEAR_PRIORITY,"dummy",0) ) {
   Ierror(E_CMD_NOT_SEND,"CHANNELS","");
    return;
  }
  if  (!sendCommand(CLEAR_DEVICES,"dummy",0) ) {
   Ierror(E_CMD_NOT_SEND,"CHANNELS","");
    return;
  }
  sendAllPriorities();
  if (!sendCommand(PRIORITY_CHECK,"dummy",0)) {
     printf("ERROR : MAX could not send PRIORITY_CHECK command \n");
  }	
  usleep(50);	
  sendAllDeviceData();
  in_fname   -> setText(max_init_filename);
  prio_fname -> setText(pr_filename);
  dev_fname  -> setText(init_filename);
}

void MainGUI::reinit_priority() {
	device_lb_flag = FALSE; //device listbox will be cleared and refilled
	if (Priority.Pr_initlist(pr_filename)==0) {
		printf("Error while reloading priority file '%s'\n",
				pr_filename);
    		printf("I will quit the applications \n");
		if (!sendCommand(KILL_YOURSELF,"dummy",0)) {
			printf("ERROR : MAX could not send kill command \n");
      			return;
    		}	
    		QApplication::exit();
  	}
  	priority_update();
  	if  (!sendCommand(CLEAR_PRIORITY,"dummy",0) ) {
		Ierror(E_CMD_NOT_SEND,"CHANNELS","");
    		return;
  	}
  	sendAllPriorities();
  	if (!sendCommand(PRIORITY_CHECK,"dummy",0)) {
     		printf("ERROR : MAX could not send PRIORITY_CHECK command \n");
 	}	
}

void MainGUI::reinit_devices() {
	device_lb_flag = FALSE; //device listbox will be cleared and refilled
	Devices->initlist(init_filename);
	listbox_update();
	if  (!sendCommand(CLEAR_DEVICES,"dummy",0) ) {
		Ierror(E_CMD_NOT_SEND,"CHANNELS","");
		return;
  	}
	sendAllDeviceData();
}

void MainGUI::help() {
	int pid;
	if ( ( pid = fork()) < 0 ) {
		perror("Fork error, I will leave the application..");
      		exit(-1);
  	}
  	else if (pid == 0) { //child
    		//exec netscape
		if ((pid=system("netscape ./doc/index.htm" )) < 0) {
			perror("MAX : exec error for netscape  ");
		}
		exit(1);
  	}
}

//fill left listbox with device parameters
void MainGUI::listbox_update() {
  PlotProps *X;
  char *strptr;
  const char *dev_name;
  const char *var_name;
  int position;    //index of variable in device parameter list
  int lbindex  =0; //index of variable in listbox
  int lbindex2 =0;
  int pindex   =0; //index to map listbox pos to PlotPropList position
  int x;
  position = 1;
  if (!device_lb_flag) //listbox has to be cleared an refilled
    device_lb->clear();
  while ((strptr = Devices->getstring(position)) != NULL) {
    dev_name = Devices->get_name(position-1); //get dev and var name
    var_name = Devices->get_var_name(position -1);
    if (dev_name && Priority.Get_priority(Devices->get_name(position-1))) {
      //only add devices with priority != 0 to the list
      //scan through plotlist and try to find this name
      x=0;
      for (X=Devices->PlotPropList -> first();
	   X != NULL;X = Devices->PlotPropList -> next()) {
	if ( (strcmp(dev_name,X -> dev_name.c_str()) ==0) )
	  if (strcmp(var_name,X -> var_name.c_str()) ==0)  {
	    //build new string now insert comment for plotdataflag
	    memset(strptr,'\0',strlen(strptr)); //erase string
	    sprintf(strptr,"%-10s : Store : %-10s",dev_name,var_name);
	    if (X ->storeflag) //is it enabled
	      strcat(strptr," enabled ");
	    else
	      strcat(strptr," disabled");
	    if (X -> plotflag)
	      strcat(strptr," Plot");
	    else
	      strcat(strptr," NoPlot");
	    X = NULL;
	    pindex = x;
	  }
	x++;
      }
      if (device_lb_flag) //if lb is not empty only replace values
	device_lb -> changeItem(strptr,lbindex2++);
      else   //lb is empty fill it
	device_lb -> insertItem(strptr);
      lbtodata[lbindex]       =position-1;//map listboxposition to Database position
      lbtoplotprops[lbindex++]=pindex;  //map listboxpos. to PlotDatabase pos.
    }
    position++;
  }
device_lb_flag = TRUE;
}

void MainGUI::priority_update(){
  int pr,position = 0;
  QString str;
  QString numstr;
  prio_lv -> clear();
  while((str = Priority.Get_device_name(position))) {
    pr = Priority.Get_priority(position++);
    numstr.sprintf("%d",pr);
    (void) new QListViewItem( prio_lv,(const char *)  str, (const char *) numstr );
  }
}

void MainGUI::set_plotwindow_update_intervall() {

  W->Setupdate(plot_update, data_send_intervall);
  W  -> show();
}

void MainGUI::set_trigger_internal() {
  if (!m_running) {
    if  (!sendCommand(INT_TRIGGER,"a",1) ){
      Ierror(E_CMD_NOT_SEND,"CONTINUE","");
      return;
    }
    button_triggermode -> setText("Trigger Mode : INTERNAL");
    trigger_mode = FALSE;  //set trigger internal
  }
}

void MainGUI::set_trigger_external() {
  if (!m_running) {
    if  (!sendCommand(EXT_TRIGGER,"a",1) ){
      Ierror(E_CMD_NOT_SEND,"CONTINUE","");
      return;
    }
    button_triggermode -> setText("Trigger Mode : EXTERNAL");
    trigger_mode = TRUE;
  }
}

void MainGUI::set_trigger_delay() {
  DelayW->Setdelay(trigger_period);
  DelayW->show();
}

void MainGUI::toggle_save_widget_enable() {
  if (!save_detect_enable) {
    printf("MAX : Save widget will automatically be shown after measurement has stopped. \n");
    save_detect_enable = TRUE;
  }
}

void MainGUI::toggle_save_widget_disable() {
  if (save_detect_enable) {
    printf("MAX : Save widget will never be shown automatically. \n");
    save_detect_enable = FALSE;
  }
}

//send trigger delay to polld
void MainGUI::apply_trigger_delay(uint period) {
  if  (!sendCommand(SET_PERIOD,"a",period ) ){
    Ierror(E_CMD_NOT_SEND,"CONTINUE","");
    return;
  }
  trigger_period = period;
}

void MainGUI::apply_update_intervall(uint period,uint pl) {
  plot_update = period;
  data_send_intervall=pl;
  update_timer -> changeInterval(period);
  if (!sendCommand(SET_DAT_SEND_INTERVALL,"dummy",data_send_intervall) ) {
    Ierror(E_CMD_NOT_SEND,"SET_DATA_SEND_INTERVALL","");
    return;
  }
}

void MainGUI::apply_options(int enable,int plot) {
  PlotProps *X;
  X=Devices->PlotPropList ->at(plot_index);
  X -> plotflag = plot;
  X -> storeflag = enable;
  Devices->set_enabled_flag(dev_index,enable);
  Devices->set_plot_flag(dev_index,plot);
  listbox_update();
  //  printf("mainGUI aplying so %d plo %d\n",enable,plot);
}

void MainGUI::select_plotvars() {
  printf("select plotvars\n");
}

void MainGUI::set_maxnum() {
	QString str;
  	str = channel_editor->text();
  	if  (!sendCommand(CHANNELNUMBER,"dummy",atoi(str)) ) {
    		Ierror(E_CMD_NOT_SEND,"CHANNELS","");
    		return;
  	}
  	max_channels = atoi(str);
}

void MainGUI::set_integrations(){
  	QString str;
  	str = integ_editor->text();
  	if  (!sendCommand(INTEGRATIONS,"a",atoi(str)) ) {
    		Ierror(E_CMD_NOT_SEND,"INTEGRATIONS","");
    		return;
  	}
  	max_integrations = atoi(str);
}

/*
 *listbox item is activated and shown in edit-box
 *lbtodata maps listbox position to actual position in the database
 *one could (should) usr array of pointers to listelements -> sorting ?
 */
void MainGUI::edit_parameter(int lb_index) {
  QString str;
  PlotProps *X;
  dev_index = lbtodata[lb_index];       //get database index
  if (Devices->get_store_flag(dev_index)) {        //check if store variable and no parameter
    if (!m_running) {
      plot_index = lbtoplotprops[lb_index]; //get PlotPropertiesList index
      X=Devices->PlotPropList ->at(plot_index);
      Options *O = new Options( X -> dev_name.c_str(),X -> storeflag,X->plotflag);
      connect(O  ,SIGNAL(SIG_setoptions(int,int ))
	      ,this, SLOT( apply_options(int,int)));
      O -> show();
    }
  }
  else { //regular parameter -> edit
    switch (Devices->get_type( dev_index )) {
    case 1:  //int
      str.sprintf("%d",Devices->getint( dev_index ));
      break;
    case 2: //double
      str.sprintf("%f",Devices->getdouble( dev_index ));
      break;
    case 3:
      str.sprintf("%c",Devices->getchr( dev_index ));
      break;
    case 4:
      str.sprintf("%s",Devices->getstr ( dev_index ));
      break;
    }
    //"switch on " editor window
    dev_lb_update = FALSE;
    dev_editor -> setPalette(QPalette (grp_active,grp_active,grp_active));
    dev_editor -> setText( str );
    dev_editor -> selectAll();
    dev_editor -> setFocus();
  }
}

//device listbox :
//after return is pressed this function is called
void MainGUI::set_new_parameter( ) {
  int type;
  unsigned char enabled;
  QString buf;
  unsigned char *name = new unsigned char[ MAX_NAME_LENGTH ];
  unsigned char *var  = new unsigned char[ MAX_NAME_LENGTH ];
  int index = -1;
  //switch off editor window
  dev_editor -> setPalette(QPalette (grp_passive,grp_passive,grp_passive));
  dev_editor -> clearFocus();
  int dindex; //index of element in database
  //only edit var if listboxitem is activated
  if ((index = device_lb->currentItem())!=-1) {
    dindex = lbtodata[index]; //get index of elemement id device_data base
    buf = dev_editor->text(); //save copy of text assign const char * to char *
    type = Devices->get_type(dindex); //check for type of var
    name = (unsigned char *) Devices->get_name    (dindex); //get devices name
    var  = (unsigned char *) Devices->get_var_name(dindex); //get var name
    enabled = (unsigned char ) Devices->get_enabled_flag(dindex); //get enabled flag
    switch (type) {
    case 1: //write int var into database and send to polldeamon
      sendDeviceData(name,var,(unsigned char *) qstrdup(buf),'1',enabled);
      Devices->putvar(atoi(qstrdup(buf)),dindex);
      break;
    case 2:
      sendDeviceData(name,var,(unsigned char *) qstrdup(buf),'2',enabled);
      Devices->putvar( atof(qstrdup(buf)),dindex);
      break;
    case 3:
      sendDeviceData(name,var,(unsigned char *) qstrdup(buf),'3',enabled);
      Devices->putvar( qstrdup(buf),dindex);
      break;
    case 4:
      sendDeviceData(name,var,(unsigned char *) qstrdup(buf),'4',enabled);
      Devices->putvar( qstrdup(buf),dindex);
      break;
    }
    dev_editor->setText(" ");
    listbox_update();
    dev_lb_update = TRUE;
  }
}

//edit the priority value in the LineEditbox
void MainGUI::edit_priority(QListViewItem *index) {
  if (!m_running) {
    prio_editor -> setPalette(QPalette (grp_active,grp_active,grp_active));
    QString str;
    str.sprintf("%d", atoi(index->text(1))); //get text from column 1
    prio_editor -> setText( str );
    prio_editor -> selectAll();
    prio_editor -> setFocus();  //set cursor in QLineEdit
  }
}

//set new value of priority in priority list
//after return is pressed in LineEdit for priorities
void MainGUI::set_priority() {
  device_lb_flag = FALSE; //device listbox will be cleared and refilled
  QListViewItem *item;
  item = prio_lv -> currentItem(); //points to Item currently highlighted
#ifdef _QT2_
  	QString text = item->text(0);
  	char *name = (char *) text.latin1();
#else	
  	char *name = (char *) item -> text(0);   //get device name from column zero
#endif  
  QString str;
  prio_editor -> setPalette(QPalette (grp_passive,grp_passive,grp_passive));
  prio_editor -> clearFocus();   //remove cursor from QlineEdit Widget
  if (!m_running) {             //only edit if one element is selected
    str = prio_editor -> text();        //copy value in buffer
    Priority.Pr_append(name,atoi(str)); //copy val. from lineedit into database
    //printf("device is %s ,value %d \n",name,atoi(str));
    prio_editor -> setText (" ");             //clear editbox
    priority_update();
    listbox_update();
    SortPlotProperties();      //update priorities in the plot Properites list
  }
}

//Show popUp Menu if right Mouse Button is clicked
void MainGUI::prio_lv_PopUp(QListViewItem *item,const QPoint &pos,int column) {
  prio_lv -> setSelected(item,TRUE);
  priority_trigger -> popup(pos,column);
}

//SLOTS of file name edit boxes
void MainGUI::Prio_Enabled() {
  	int max_prio;
  	QListViewItem *it = prio_lv -> currentItem();
#ifdef _QT2_
  	QString text = it->text(0);
  	char *name = (char *) text.latin1();
#else	
  	char *name = (char *) it -> text(0);   //get device name from column zero
#endif  
  	if ( ( Priority.Get_priority(name)) != 0) { //prio is already enabled
    		prio_lv -> setSelected(it,FALSE); //deselect it
    		return;
  	}
  	else {
    		max_prio = Priority.Max_Priority();
    		Priority.Pr_append(name,max_prio+1);
    		device_lb_flag = FALSE; //device listbox will be cleared and refilled
    		priority_update();
    		listbox_update();
    		SortPlotProperties();      //update priorities in the plot Properites list
 	}
}

void MainGUI::Prio_Disabled() {
  QListViewItem *it = prio_lv -> currentItem();
#ifdef _QT2_
  	QString text = it->text(0);
  	char *name = (char *) text.latin1();
#else	
  	char *name = (char *) it -> text(0);   //get device name from column zero
#endif  
  Priority.Pr_append(name,0);
  device_lb_flag = FALSE; //device listbox will be cleared and refilled
  priority_update();
  listbox_update();
  SortPlotProperties();      //update priorities in the plot Properites list
}

void MainGUI::edit_pfile() {
  strcpy(pr_filename,prio_fname->text());
  prio_fname -> clearFocus();
  prio_fname -> setPalette(QPalette
			   (grp_passive,grp_passive,grp_passive));
}

void MainGUI::edit_dfile() {
  strcpy(init_filename,dev_fname->text());
  dev_fname -> clearFocus();
  dev_fname -> setPalette(QPalette
			  (grp_passive,grp_passive,grp_passive));
}

void MainGUI::edit_ifile() {
   strcpy(max_init_filename,in_fname -> text());
in_fname -> clearFocus();
  in_fname -> setPalette(QPalette
			 (grp_passive,grp_passive,grp_passive));
}

//timer event called to make pause button blink
void MainGUI::ButtonTimerEvent(void) {
  if (pause_flag) //restart timer only if button is pressed
    blink_timer -> start(BUTTON_BLINK, TRUE );
  if (color_state == 1) {
    pbpause ->setPalette( QPalette(colgrp_off,colgrp_off,colgrp_off) );
    color_state = 0;
  }
  else {
    pbpause ->setPalette( QPalette(colgrp_on,colgrp_on,colgrp_on) );
    color_state = 1;
  }
}

//timer event called to refresh the channel number shown in the main widget
void MainGUI::RefreshNumber(void) {
  if (dev_lb_update )
      listbox_update();
}

//
//this routine empties the plot data queue
//during the first channel it initializes the plotwindowd and copies pointers
//to the measured data into the appropriate windows
//after the plotdata queue is emptied the windows receive the update signals to
//deep copie the data and show it on the screen
//this way, only the latest data in the queue is visible
//a stack stack is not usable here, because scalar data have to be copied completely
//when arrays are plotted, only the last one is interesting
//
void MainGUI::UpdatePlotWindows(void ) {
  bool flag=TRUE;
  int act_prio=0;  		//actual priority
  int numberstores=0;
  if (! P_Buf -> count() )     //only do something if data present
    return;
  PlotProps 	*X;
  PlotBuffer 	*P;
  for (X=Devices->PlotPropList -> first(); X != NULL;X = Devices->PlotPropList -> next())
    if (X -> priority != 0)
      numberstores ++;
  if (varcounter+arraycounter > numberstores) {
    update_timer -> stop();
    char errmsg[450];
    if  (!sendCommand(STOP,"a",1) ) {
      Ierror(E_CMD_NOT_SEND,"STOP","");
      return;
    }
    m_running = FALSE;
    pause_flag = FALSE; //stop pause
    chn_label -> setText("Stop");
    sprintf(errmsg,"Missmatch between the STORE statements of the Device Parameter Initfile and the measured data.\nThere are %d datasets per channel 'stored' by your devices.\nBut %d store commands in the Initlist\nI can not handle this. Check the store & enable statements in your device function\nand your Device Parameter Initfile.\nMaybe you have forgotten an enabled statement of your device function or a STORE statement in the Initfile.\nI will terminate this application now.",varcounter,numberstores);
    QMessageBox::query("ATTENTION!!",errmsg,"OK");
    if (!sendCommand(KILL_YOURSELF,"dummy",0)) {
      printf("MAX : ERROR. Could not send kill command \n");
      return;
    }
    kill(pid,SIGKILL);
    QApplication::exit();
  }
  for (X =Devices->PlotPropList -> first();X != NULL;
       X =Devices ->PlotPropList -> next()) {
    X -> marker = FALSE;
  }
  //this routine looks at one element of the PlotBuffer List
  //the goes through the whole PlotProps list and stops as soon as the first element is found
  // for (P= P_Buf -> first(); P != NULL; P = P_Buf ->stop_cl next() ) {
  //  printf("channel %d, ind %d %d\n",P -> chann(),P -> ind(),P->ival());}
  X = Devices->PlotPropList -> first();
  for (P= P_Buf -> first(); P != NULL; P = P_Buf -> next() ) {
    X = Devices->PlotPropList -> first();
    if (P->chann()==1) {
      WindowPtr.push_back(-1);          //add element to vector, no plot Window index yet
      //      printf("chan = 1 : size %d \n",WindowPtr.size());
      windowcounter = 0;
      flag = TRUE; 			//flag used to quit while loop if a windoe has been opened
      while (flag && X ) {  		//loop over all elements of X
	if (X -> priority == P -> prior() &&
	    X  -> storeflag == 1 && !(X -> marker) ) {
	  if ( X -> plotflag  == 1   ) {   //also plot it ?
	    //	    printf("plot it %s \n",X-> var_name.c_str());
	    Plot * win = new Plot(0,"plotwin",(char *)X -> dev_name.c_str(),
				  (char *) X -> var_name.c_str(),
				  max_channels,monitor_mode);
	    //printf("plot value %d \n",P->ival());
	    SetWindowParameters(win,X);        	//send all options to this window
	    WindowList ->append(win);		//append pointer to the list
	    WindowList ->current()->show();     //show it
	    WindowPtr[WindowPtr.size()-1]=WindowList -> count()-1; //index x contains position of window in lisr
	    switch(P  -> var_type()) {
	    case 1: //int
	      if (P -> len() == 1)
		WindowList ->current()->plotPoint(P -> chann(),P->ival());
	      else
		WindowList ->current()->plotPoint(P ->chann(),P ->ivec(),P -> len());
	      break;
	    case 2: //double
	      if (P -> len() == 1)
		WindowList ->current()->plotPoint(P -> chann(),P->dval());
	      else
		WindowList ->current()->plotPoint(P -> chann(),P ->darr(),P -> len());
	      break;
	    }
	    flag = FALSE;
	    act_prio = P -> prior();	     //copy priority of last element
	    X -> marker = TRUE;		     //set marker don't take it again
	  }
	  else { //plotflag is zero simply move Plotbuffer to next element
	   P = P_Buf -> next();
	   WindowPtr.push_back(-1);          //add element to vector, no plot Window index yet
	  }
	}	
	X = Devices->PlotPropList -> next(); //check next element	
      }
    }
    else {		 //channel > 1 , //when new channel is reached reset some pointers
      if (WindowPtr[P -> ind()]> -1) {         //if not -1 then this index has a plotWindow
	switch(P -> var_type()) {
	case 1: 			//int
	  if (P -> len() == 1)
	    WindowList ->at(WindowPtr[P -> ind()])->plotPoint(P -> chann(), P->ival());
	  else
	    WindowList ->at(WindowPtr[P -> ind()])->plotPoint(P -> chann(),
							      P-> ivec(),P->len());
	  break;	
	case 2: 			//double
	  if (P -> len() == 1)
	    WindowList ->at(WindowPtr[P -> ind()])->plotPoint(P -> chann(),P->dval());
	  else
	    WindowList ->at(WindowPtr[P -> ind()])->plotPoint(P -> chann(),
							      P->darr(),P -> len());
	  break;
	}
      }
    }  //of else chan > 1
    act_prio = P -> prior(); //copy priority of last element
  }
  Plot *pp; //pointer to plotwindow in WindowList
  for (pp = WindowList -> first(); pp != NULL; pp = WindowList -> next() ) {
    pp -> refresh();
  }	
  P_Buf -> clear();
  if (monitor_mode)
    Schedule -> remove();
  delete pp;
}

//set all options out of th plotPorperities List
//in the speceific Plotwindow
void MainGUI::SetWindowParameters(Plot *P,PlotProps *X) {
  P -> SetAllOptions(&(X -> Ps));
}

//add the actual priorites to the plot properties list
//this funtion is called right before the measurement starts
//needed to determine the individual plotwindows
//for each measured datapoint(array)
void MainGUI::SortPlotProperties(void) {
 PlotProps *P;
 for (P = Devices->PlotPropList -> first();P != NULL;
      P = Devices->PlotPropList-> next() ) {
   P->priority= Priority.Get_priority((char *) P-> dev_name.c_str());
 }
}

/*!
 *This routine is called after the START button is pressed
 *It clears  the local database,
 *sends all device data to the Polling Deamon
 *and sends the START command
 */
void MainGUI::startclicked() {
  int l=0;
  bool ok=FALSE;
  if (!m_running) {
    if (Schedule -> database_empty() || Schedule -> array_empty()) { //data Present ?
       if (!save_flag ) { //Data present, no yet saved
	 ok = QMessageBox::query("ATTENTION!!",
				 "Data no saved yet, Do it now ?","YES","NO");
       }
    }
    if (!ok) {
      if ((l=Priority.Consistency()) == 0) {
	ok = QMessageBox::query("ATTENTION!!",
				 "     Missing Priorities found !\nYour Priority database is inconsistent\n    Please Check and start again.","OK");
    	return;
      }
      if (Priority.check4double()==0) {
	ok = QMessageBox::query("ATTENTION!!",
				 "     Double Priorities found !\nYour Priority database is inconsistent\n    Please Check and start again.","OK");
    	return;
      }
      if (!Priority.check4zerolist()) {
		ok = QMessageBox::query("ATTENTION!!",
				 "   There is no device with Priority 1 !\nCheck your priority database and start again.","OK");
    	return;
      }
      delete P_Buf;
      P_Buf = new QList<PlotBuffer>;
#ifdef _GCC_
      WindowPtr.erase(WindowPtr.begin(),WindowPtr.end()); //clear vector with indices of WindowList
#endif
#ifdef _EGCS_
      WindowPtr.clear();
#endif
      Schedule   -> remove(); //clear databases
      WindowList -> clear();  //remove all plotwindows
      SortPlotProperties();   //get list of all plotvariables from initfile
      if (VarListPtr)
	delete [] VarListPtr;
      if ( ArrayListPtr )
	delete [] ArrayListPtr;
      VarListPtr       = new VarType[ACTIVE_VARIABLES];
      assert(VarListPtr != 0);
      memset(VarListPtr,0,sizeof(VarListPtr));
      ArrayListPtr     = new ArrayType[ACTIVE_VARIABLES];
      assert(ArrayListPtr != 0);
      memset(iarraybuffer,0,MAX_ARRAY_SIZE);
      memset(darraybuffer,0,MAX_ARRAY_SIZE);

      pause_flag = FALSE;
      //flush database of polling deamon and refill it
      if (sendCommand(CLEAR_DEVICES,"a",1) == 0) {
	Ierror(E_CMD_NOT_SEND,"CLEAR_DEVICES","");
	return;
      }
      if (sendCommand(CLEAR_PRIORITY,"a",1) == 0) {
	Ierror(E_CMD_NOT_SEND,"CLEAR_PRIORITY","");
	return;
      }
      // usleep(50); 	 //wait for preceeding commands to be received
      sendAllDeviceData(); //send new data to refill database
      //  usleep(50);  	 //wait to make shure all data is properly received
      sendAllPriorities();
      if (!sendCommand(PRIORITY_CHECK,"dummy",0)) {
	printf("ERROR : MAX could not send PRIORITY_CHECK command\n");
      }	
      update_timer -> start( plot_update, FALSE ); //always runs
      if (!monitor_mode)
	save_flag = FALSE;
      else
	save_flag = TRUE;

      arraycounter = 0;
      varcounter   = 0;
      bufpos       = 0;

      //set Progress Bar for channel numbers
      if (!monitor_mode )
	progress -> setTotalSteps( max_channels );
      else
	progress -> setTotalSteps( MON_MODE_CHANNELS );
      progress -> reset(  );

      //set Progress-bar for integrations

      int_progress -> setTotalSteps( max_integrations );
      int_progress -> reset(  );

      if (sendCommand(START,"a",1) == 0) {
	Ierror(E_CMD_NOT_SEND,"START","");
	return;
      }
      m_running = TRUE;
      chn_label -> setText("Running..");
    }
    else  //else of save data warning box
      save();
  }
}

//the continue button is clicked
//send command to server and erase lokal buffer
void MainGUI::contclicked(){
  if (!m_running) {
    delete P_Buf;
    P_Buf = new QList<PlotBuffer>;
    if  (!sendCommand(CONTINUE,"a",1) ) {
     Ierror(E_CMD_NOT_SEND,"CONTINUE","");
      return;
    }
    m_running = TRUE;
  }
}

void MainGUI::stopclicked(){
  if  (!sendCommand(STOP,"a",1) ) {
    Ierror(E_CMD_NOT_SEND,"STOP","");
    return;
  }
  m_running = FALSE;
  pause_flag = FALSE; //stop pause
  chn_label -> setText("Stop");
}

void MainGUI::pauseclicked() {
  if (m_running ) {
    if (!pause_flag) {
      if  (!sendCommand(PAUSE,"a",1) ) {
	Ierror(E_CMD_NOT_SEND,"PAUSE","");
	return;
      }
      pause_flag = TRUE;
      blink_timer -> start(500, TRUE );
      chn_label -> setText("Pause..");
    }
    else {
      if  (!sendCommand(PAUSE,"a",1) ) {
	Ierror(E_CMD_NOT_SEND,"PAUSE","");
	return;
      }
      pause_flag=FALSE;
      if (!m_running )
	 chn_label -> setText("Stop");
      else
	chn_label -> setText("Running..");
    }
  }
}

void MainGUI::resetclicked() {
  if (!m_running) {
    if  (!sendCommand(RESET,"a",1) )
    Ierror(E_CMD_NOT_SEND,"REST","");
    return;
  }
}

void MainGUI::toggle_trigger_mode() {
  if (!m_running)                          //no measurement running
    if (trigger_mode) {          //external trigger -> set internal
      if  (!sendCommand(INT_TRIGGER,"a",1) ){
	Ierror(E_CMD_NOT_SEND,"CONTINUE","");
	return;
      }
      button_triggermode -> setText("Trigger Mode : INTERNAL");
      trigger_mode = FALSE;  //set trigger internal
    }
    else {
      if  (!sendCommand(EXT_TRIGGER,"a",1) ){
	Ierror(E_CMD_NOT_SEND,"CONTINUE","");
	return;
      }
      button_triggermode -> setText("Trigger Mode : EXTERNAL");
      trigger_mode = TRUE;
    }
}

void MainGUI::toggle_monitor_mode() {
  if (!m_running) {
    if (monitor_mode) { //TRUE = in monitor mode
      if  (!sendCommand(DATAQ_MODE,"a",1) ){
	Ierror(E_CMD_NOT_SEND,"DATAQ_MODE","");
	return;
      }
      button_monitormode -> setText("Data Aquisition Mode");
      monitor_mode = FALSE;
      Schedule -> set_MonMode(monitor_mode);  //send info to database
    }
    else {
      if  (!sendCommand(MON_MODE,"a",1) ){
	Ierror(E_CMD_NOT_SEND,"MONITOR_MODE","");
	return;
      }
      button_monitormode -> setText("    Monitor Mode     ");
      monitor_mode = TRUE;
      Schedule -> set_MonMode(monitor_mode); //send info to database
    }
  }
}

//event handler
bool MainGUI::eventFilter( QObject *obj,QEvent *event) {
#ifdef _QT2_
	int Event_MouseButtonPress = QEvent::MouseButtonPress;
#endif
	
  if (event -> type() ==
      Event_MouseButtonPress && strcmp(obj->name(),"channel_editor")==0) {
    channel_editor -> setPalette(QPalette (grp_active,grp_active,grp_active));
  }
  if (event -> type() ==
      Event_MouseButtonPress && strcmp(obj->name(),"integrations_editor")==0) {
    integ_editor -> setPalette(QPalette (grp_active,grp_active,grp_active));
  }
  if (event -> type() ==
      Event_MouseButtonPress && strcmp(obj->name(),"priority_editor")==0) {
    prio_fname -> setPalette(QPalette (grp_active,grp_active,grp_active));
  }
  if (event -> type() ==
      Event_MouseButtonPress && strcmp(obj->name(),"dev_fname_editor")==0) {
    dev_fname -> setPalette(QPalette (grp_active,grp_active,grp_active));
  }
  if (event -> type() ==
      Event_MouseButtonPress && strcmp(obj->name(),"init_fname_editor")==0) {
    in_fname -> setPalette(QPalette (grp_active,grp_active,grp_active));
  }
  return FALSE; //don't eat event
}

/****************************************
 *this slot is called when server sends
 *device Parameter data on the data_socket
 *the name is wrong
 *it should be parameterreceived
 ****************************************/
void MainGUI::dataReceived(void) {
  int data_available=0;
  int i_value=0;
  int enabled;
  double d_value = 0;
  devDataPacket *d_packet = new devDataPacket;
  while ((data_available = recv(devr_sfd,(char *) d_packet,DPACKETSIZE,MSG_PEEK)) == DPACKETSIZE) {
    if ((n=recv(devr_sfd,(char *) d_packet,DPACKETSIZE,0)) < 0 ) {
      perror("MAX : ERROR DevDataReceiuve  ");
      return;
    }
    //  printf("data packet received %s %s \n",(char *) d_packet -> device_name,
    //	   (char *) d_packet -> var_name);
    enabled = (int ) d_packet -> enabled;
    if (d_packet -> type == '1' ) {
      i_value = strtol((const char *) d_packet -> value,NULL,10);
      Devices->append((int) i_value,(char *) d_packet -> device_name,
		      (char *) d_packet -> var_name,enabled,1,0,0);
    }
    if (d_packet ->type == '2') {
      d_value = strtod((const char *) d_packet -> value,NULL);
      Devices->append((double) d_value,(char *) d_packet -> device_name,
		      (char *) d_packet -> var_name,enabled,2,0,0);
    }
  }
  //only update plot listbox if no editing is going on
  //-> dev_lb_update flag
  if (!number_timer -> isActive() && dev_lb_update ) {
    number_timer -> start( update_intervall, TRUE );
    dev_lb_update = TRUE;
  }
  delete d_packet;
  return;
}

//Main GUI receives a command from polld server through this socket :
//these commands are  : act. channel number & integrations, stop,error ...
void MainGUI::commandReceived(void) {
  	if ( RCmdNotifier->type() == QSocketNotifier::Exception) {
		cout<<"MAX : Exception in Command Received SocketNotifier Event\n";
    		return;
  	}
  	
	command_packet *c_packet = new command_packet;
  	cmdClient_max_len        = sizeof(cmdClient_max);
  	if ((n = read(cmdr_sfd, c_packet, CPACKETSIZE )) <= 0 ) {
#ifdef DEBUG_MAX
  		printf("MAX : recv(cmdr_sfd) = %d!\n",n);
#endif
    		perror("MAX : Couldn't read from cmdr_sfd\nMAX : POLLD
				may be died !!!");
    		RCmdNotifier -> setEnabled(FALSE); //Error on socket, disable notifier
    		QMessageBox::query("ATTENTION!!","Error while trying to read from Command Socket!\nThe Pipe to the Server is propably broken.(Did the Server terminate abnormally ?)\nSave all data now and then you should leave MAX and restart the application.","OK");
    		save();
    		return;
	}
  	// commands see definitions.h
  	// printf("MAX : recv (%d):  command = %c OK\n", n, 
	//	c_packet->command);
  	memcpy(&cmdvalue,c_packet->data,sizeof(int));
	switch ( c_packet -> command ) {  //parse command field
	case START :
		INFO("MAX : Starting Measurement\n");
    		break;
	case STOP :
		INFO("MAX : STOP\n");
		stop_flag = 1;
    		m_running = 0;
    		chn_label -> setText("Stop");
    		if (!monitor_mode)
      			if (save_detect_enable)
		save();
    		break;
	case PAUSE :
		INFO("MAX : PAUSE\n");
		pause_flag = 1;
		break;
	case CONTINUE :
		INFO("MAX : CONTINUE\n");
    		break;
  	case RESET :
		INFO("MAX : RESET\n");
    		break;
  	case CHANNELNUMBER :  //client sets channelnumber
    		chan_no -> display(cmdvalue);
		// in monitor mode reset progress bar after MON_MODE channels
    		if (monitor_mode ) 
      			if (cmdvalue < progress -> progress() ) // channel flipped already
				progress -> reset();//reste to zero
      		progress -> setProgress( cmdvalue);
    		break;
	case INTEGRATIONS :   //client sets integrations
		if (int_progress -> progress() > cmdvalue )
			int_progress -> reset(); //reset if integ. is for next channel
    		int_progress -> setProgress ( cmdvalue);
		break;
	case MON_MODE :      //client sets MONITOR MODE
		INFO("MAX : Main GUI : Monitor Mode  \n");
		break;
	case DATAQ_MODE :    //client sets Data Acqusition mode
		INFO("MAX : Data Aqusisition Mode\n");
		break;
	case PRIORITY :       //priority added or changed by GUI
		INFO("MAX : Priority signal received \n");
		break;
	case EXT_TRIGGER :
		INFO("MAX : External Trigger \n");
		break;
	case INT_TRIGGER :
		INFO("MAX : Internal Trigger \n");
		break;
	case SET_PERIOD :
		INFO("MAX : Set Period\n");
		break;
	case CLEAR_DEVICES :
		INFO("MAX : Clear Devices\n");
		break;
	case CLEAR_PRIORITY :
		INFO("MAX : Clear Priorities\n");
		break;
	case ERROR :
		INFO("MAX : Received Error Message !!! \n");
		bool ok = Ierror((char * ) c_packet -> error_msg,
				(char *) c_packet -> name,"");
		sendCommand(ERROR,"bla",ok); //send answer back
  	}
	delete c_packet;
}

/**************************
 * MEASURED DATA received slot
 * (name is wrong)
 * this slot is called when server (the polling deamon)
 * sends the measured data to the client
 **************************/
void MainGUI::parameterReceived( void ) {
  int data_available;
  memset(&packet,0,FULL_P_SIZE);// erase packet
  while ((data_available = recv(mdat_sfd,(char *) &packet,FULL_P_SIZE,MSG_PEEK)) == FULL_P_SIZE) {
    if ((n=recv(mdat_sfd,(char *) &packet,FULL_P_SIZE,0)) < 0) {
	perror("MAX : Error. Client data received");
	return;
    }
    else
      appendData(packet); //check data and append it to the database	
  }		
}

void MainGUI::appendData(dataPacket packet) {
    char type;
    int dat_length;
    int t_length;
    int index,chn,prio;
    int *p;
    memcpy(&index,     &(packet.var_index)  ,sizeof(int));
    //       printf("appending index %d \n",index);
    memcpy(&prio,      &(packet.priority)   ,sizeof(int));
    memcpy(&chn,       &(packet.channel)    ,sizeof(int));
    memcpy(&t_length,  &(packet.t_length)   ,sizeof(int));
    memcpy(&dat_length,&(packet.data_length),sizeof(int));
    if (  (type = packet.type) == 'a') { //int
	memcpy(buf,&(packet.buffer),DATA_PACKET_SIZE);
    }
    if ( type == 'b' ) {
	memcpy(dbuf,&(packet.buffer),DATA_PACKET_SIZE);
    }
    if (chn == 1) { //during first channel counter number of scalars received
	if (type == 'a' ) {//int
	    if (dat_length == sizeof(int)  ) { //scalar
	      VarListPtr[varcounter++].type = 1; //int
	      Schedule -> set_N_Elements(varcounter); //number of elements per channel stored here
	      Schedule -> add_data(buf[0]);
	      PlotBuffer *P = new PlotBuffer(buf[0],chn,index,prio);
	      P_Buf -> append(P);  //append to plotbuffer
	    }
	    else {   //array of int //array longer than one packet ?
	      if (int (t_length-bufpos)*sizeof(int) > DATA_PACKET_SIZE){
		p = &(iarraybuffer[bufpos]);
		memcpy(p,buf,dat_length); //copy array in buffer
		bufpos+= dat_length/sizeof(int);
	      }
	      else {  //array only fills one packet -> append it
		ArrayListPtr[arraycounter].type =1;
		ArrayListPtr[arraycounter++].size = dat_length/sizeof(int);
		if (! bufpos ) {    //no data queued append right away
		  Schedule -> set_N_Arrays(arraycounter); //number of arrays per channel
		  PlotBuffer *P= new PlotBuffer(Schedule -> add_data(buf,dat_length/sizeof(int))
						,dat_length/sizeof(int),chn,index,prio);
		  P_Buf -> append(P);  //append to plotbuffer	
		}
		else { //add last packet from queue and append the buffer
		  memcpy(&(iarraybuffer[bufpos]),buf,dat_length);
		  Schedule -> set_N_Arrays(arraycounter); //number of arrays per channel
		  PlotBuffer *P= new PlotBuffer(Schedule -> add_data(iarraybuffer,t_length)
						,t_length,chn,index,prio);
		  P_Buf -> append(P);  //append to plotbuffer
		  bufpos = 0;
		}
	      }
	    }
	}
	else {    //double
	  if (dat_length == sizeof(double)) { //SCALAR variable
	    VarListPtr[varcounter++].type = 2;  //double
	    Schedule -> set_N_Elements(varcounter);  //number of elements stored per channel
	    Schedule -> add_data(dbuf[0]);
	    PlotBuffer *P= new PlotBuffer(dbuf[0],chn,index,prio);
	    P_Buf -> append(P);  //append to plotbuffer
	  }
	  else { //ARRRAY of double
	    //	printf("double array \n");//array longer than one packet
	    if (int (t_length-bufpos)*sizeof(double) > DATA_PACKET_SIZE){
	      double *p = &(darraybuffer[bufpos]);
	      memcpy(p,dbuf,dat_length); //copy array in buffer
	      bufpos+= dat_length/sizeof(double);
	    }
	    else {  //array only fills one packet -> append it
	      ArrayListPtr[arraycounter].type =2;
	      ArrayListPtr[arraycounter++].size = dat_length/sizeof(double);
	      if (! bufpos ) {        //no data queued append right away
		Schedule -> set_N_Arrays(arraycounter);
		PlotBuffer *P= new PlotBuffer(Schedule -> add_data(dbuf,dat_length/sizeof(double))
					      ,dat_length/sizeof(double), chn,index,prio);
		P_Buf -> append(P);  //append to plotbuffer
      	      }
	      else { //add last packet and append the buffer
		memcpy(&(darraybuffer[bufpos]),dbuf,dat_length);
		Schedule -> set_N_Arrays(arraycounter);
		PlotBuffer *P= new PlotBuffer(Schedule -> add_data(darraybuffer,t_length),
					      t_length,chn,index,prio);
		P_Buf -> append(P);  //append to plotbuffer
		bufpos = 0;
	      }
	    }
	  }
	}
    }
    else {   //CHANNEL > 0
      //printf("arraycounter %d\n",arraycounter);
      if (type == 'a' ) {                    //int
	if (dat_length == sizeof(int)  ) { //scalar
	  PlotBuffer *P = new PlotBuffer(buf[0],chn,index,prio);
	  P_Buf -> append(P);  //appen to plotbuffer
	  Schedule -> add_data(buf[0]);
	}
	else {   //array of int//array longer than one packet
	  if (int (t_length-bufpos)*sizeof(int) > DATA_PACKET_SIZE){
	    int *p = &(iarraybuffer[bufpos]);
	    memcpy(p,buf,dat_length); //copy array in buffer
	    bufpos+= dat_length/sizeof(int);
	    // printf("array to long buffering, bufpos %d \n",bufpos);
	  }
	  else {             //array only fills one packet -> append it
	    if (! bufpos ) { //no data queued append right away
	      PlotBuffer *P= new PlotBuffer(Schedule -> add_data(buf,dat_length/sizeof(int))
					      , dat_length/sizeof(int),chn,index,prio);
	      	P_Buf -> append(P);  //append to plotbuffer
	    }	
	    else { //add last packet and append the buffer
	      memcpy(&(iarraybuffer[bufpos]),buf,dat_length);
	      PlotBuffer *P= new PlotBuffer(Schedule -> add_data(iarraybuffer,t_length)
					    ,t_length,chn,index,prio);
	      P_Buf -> append(P);  //append to plotbuffer
	      bufpos = 0;
	    }	
	  }
	}
      }
      else {    //double
	if (dat_length == sizeof(double)) { //scalar variable
	  Schedule -> add_data(dbuf[0]);
	  PlotBuffer *P = new PlotBuffer(dbuf[0],chn,index,prio);
	  P_Buf -> append(P);  //appen to plotbuffer
	}
	else { //array of double
	  if (int (t_length-bufpos)*sizeof(double) > DATA_PACKET_SIZE){
	    memcpy (&(darraybuffer[bufpos]),dbuf,dat_length);
	    bufpos+= dat_length/sizeof(double);
	  }
	  else {  //array only fills one packet -> append it
	    if (! bufpos ) { //no data queued append right away
	      PlotBuffer *P= new PlotBuffer(Schedule -> add_data(dbuf,dat_length/sizeof(double)),
					    dat_length/sizeof(double),chn,index,prio);
	      P_Buf -> append(P);  //append to plotbuffer
	    }
	    else { //add last packet and append the buffer
	      memcpy(&(darraybuffer[bufpos]),dbuf,dat_length);
	      PlotBuffer *P= new PlotBuffer(Schedule -> add_data(darraybuffer,t_length),
					    t_length,chn,index,prio);
	      P_Buf -> append(P);  //append to plotbuffer
	      bufpos = 0;
	    }
	  }
	}
      }
    }	
}

//Send packet with one Device Parameter element
void MainGUI::sendDeviceData(unsigned char *dev_name,unsigned char *var_name,
			     unsigned char *value,unsigned char type,unsigned char enabled) {
  devDataPacket *sendPacket;
#ifdef DEBUG_MAX
  printf("MAX : Trying to send %s %s %s type %c\n",dev_name,var_name,value,type);
#endif
  memset(&d_packet,0,DPACKETSIZE); //erase packet
  strcpy((char *) d_packet.device_name,(char *) dev_name);
  strcpy((char *) d_packet.var_name,   (char *) var_name);
  strcpy((char *) d_packet.value,      (char *) value);
  d_packet.enabled = enabled;
  d_packet.type = type;
  sendPacket    = &d_packet;
  if (send( data_sfd,(char *) sendPacket,DPACKETSIZE,0) < 0 ) {
    perror("could not send device data ");
    Ierror(E_NOT_SEND,"device data","");
    }
}

int MainGUI::sendDeviceData(devDataPacket *sendPacket) {
  //printf("sending %10s, %10s, %c\n",sendPacket -> device_name,
  // sendPacket -> var_name,sendPacket ->type);
  if (send( data_sfd,(char *)  sendPacket,DPACKETSIZE,0) < 0) { //,
   perror("could not send device data ");
   return -1;
 }
 return TRUE;
}

void MainGUI::sendAllDeviceData( void ) {
	devDataPacket *sendPacket;
  	int endoflist = 1;
  	Devices->ResetHelpPointer();
  	do {
    		sendPacket = Devices->GetNextPacket();
    		endoflist  = Devices->AdvanceHelpPointer();
#ifdef DEBUG_MAX
    		printf("MAX : Sending \"%s %s\"\n",sendPacket -> device_name,
				sendPacket -> var_name);
#endif
	    	usleep(1);
    		if (!sendDeviceData(sendPacket)) {
      			printf("error in MainGUI::sendAllDeviceData \n");
      			return;
    		}
  	} while(endoflist);

  	INFO("MAX : All data send to POLLD\n");
}

//always send whole database
//called each time, database is reinititalized or user has changed priorites
void MainGUI::sendAllPriorities(void) {
	int endoflist=1, priority;
	char *dev_name;
  	dev_name = new  char[MAX_NAME_LENGTH];
  	assert (dev_name !=0 );
  
  	Priority.ResetHelpPointer();
  	while(endoflist) {
    		dev_name = Priority.GetNextName();
    		priority = Priority.GetNextPriority();
    		endoflist = Priority.AdvanceHelpPointer();
    		if (endoflist) { 
			//send priority data packet
       			if (! sendCommand(PRIORITY, dev_name,priority)) {
      				printf("Error in MainGUI::sendAllPriorities() \n");
      				return;
      			}
		}
  	}
  	INFO("MAX : All Priorities send to POLLD\n");
  	delete dev_name;
}

/*
 *send a command to the polld
 *definitions of commands see the file definitions.hpp
 *priorities are also send via command slot
 */
int MainGUI::sendCommand( char command, char * name, int byte ) {
  	memset(&c_packet,0,CPACKETSIZE);
  	memcpy(&c_packet.data,&byte,sizeof(int));
  	strncpy((char *) c_packet.name, name,MAX_NAME_LENGTH);
  	c_packet.command = command;
  	command_packet *send_c_packet = &c_packet;
 	INFO("MAX : Sending Command to POLLD ... ");
	int n;
  	if ((n = send(cmd_sfd ,(char *) send_c_packet,CPACKETSIZE,0)) < 0) {
   		perror("MAX : Could not send command  data");
   		return -1;
 	}
	INFO(" OK\n");
	return 1;
}

int MainGUI::reinitAll( void ) {
	int r;
#ifdef DEBUG_MAX
  	printf("MAX : max_init_filename : \"%s\"\n",max_init_filename);
#endif
  	if ( (r=readMainInitfile(max_init_filename)) == 0) {
    		QMessageBox::query("ATTENTION!!",
		       "Error while trying to read the main initfile !!\nMAX can not be properly initialized ! \nRemove the error in that file or rename it and restart the Application\nThis Application will be terminated now.\n If the Databasse has been initialized before\nI will coninue to use the old data.","OK");
    		return 0;
  	}
  	else if (r==-1) //error message has been shown in reinit routine already
      		return 0;
  		
	arraycounter = 0;
  	varcounter   = 0;
  	Schedule -> remove();	
	int result = 1;
  	stop_flag = 0;
  	result = Priority.Pr_initlist(pr_filename);
  	if (result == 0)
    		return result;
  	result = Devices->initlist(init_filename);
  	if (result==0)
    		return result;
  	//  Priority.Pr_printlist();
  	listbox_update();
  	priority_update();

	return result;
}

int MainGUI::readMainInitfile(char *filename) {
  //arguments in main initfile
  //used in MainGUI::readMainInitfle(char *filename)
	const char *initmax_parse_data[9]={"integrations","channels","dev_file",
			   "pr_file","data_file","array_file",
			   "data_dir","image_file","\0"};
 	FILE *fp;
  	for (int j=0; j<100;j++) {
    		data_dir      [j]='\0';
    		save_fname [j]='\0';
    		array_fname[j]='\0';
    		image_fname[j]='\0';
  	}
  int line = 0;
  int     stop=1;
  char    buffer[50];
  char    comment[400];
  QString str;
  int i,x,result;
  max_channels = 1 ;
  max_integrations = 1;
  char errmsg[200];
  if ((fp = fopen(filename,"r"))==NULL) {
	sprintf(errmsg,"Could not open the Main Inifile : %s\nMAX can not be initalized.",filename);
	QMessageBox::query("ATTENTION !!",errmsg,"OK");
	result = 0;
	return -1;
  }
  while (fscanf(fp,"%s",buffer) != EOF && stop == 1 ) {
    i=0;
  if (strncmp(buffer,"#",1) ==0 )       //check for comment line
           fgets(comment,400,fp);       //read until end of line
    else {
      while(strcmp(initmax_parse_data[i++],buffer) && i <9 )
	;
      line ++;
      switch(i) {
      case 1: //integrations
	  fscanf(fp,"%d",&x);
	  max_integrations = x;
	  break;
      case 2: //channels
	  fscanf(fp,"%d",&x);
	  max_channels = x;
	  break;
      case 3: //device_file name
	  fscanf(fp,"%s",init_filename);
	  break;
      case 4: //priority_filename
	  fscanf(fp,"%s",pr_filename);
	  break;
      case 5: //data filename
	  fscanf(fp,"%s",save_fname);
	  break;
      case 6: //array datafilenmae
	  fscanf(fp,"%s",array_fname);
	  break;
      case 7: //data directory path
	  fscanf(fp,"%s",data_dir);
	  break;
      case 8:
	  fscanf(fp,"%s",image_fname);
	  break;
      default :
	  printf("\n***********************************************\n");
	  printf("ERROR in the Main-initfile : %s\n",filename);
	  printf("I do not understand : %s in line %d of this file \n",buffer,line);
	  printf("***********************************************\n");
	  stop = 0;
	  break;
      }
    }
  }
  fclose(fp);
  if  (!sendCommand(INTEGRATIONS,"a",max_integrations) ) {
    Ierror(E_CMD_NOT_SEND,"INTEGRATIONS","");
    return -1;
  }
  if  (!sendCommand(CHANNELNUMBER,"channelnumber",max_channels) ) {
    Ierror(E_CMD_NOT_SEND,"CHANNELS","");
    return -1;
  }
  str.sprintf("%i",max_integrations);
  integ_editor->setText( str );
  str.sprintf("%i",max_channels); //copy int in string
  channel_editor->setText( str ); //one can only set char * in edit box
  return stop;
}

/****************************************
 * All sockets are connected in this routine
 *****************************************/
int MainGUI::connectSockets(char *hostname) {
 //First get hostname
  struct hostent *host;
  if (!(host = gethostbyname(hostname))) {
    printf("Client : Could not get hostname\n");
    return -1;
  }
  /****************************************
   *connect socket to receive measured data
   ****************************************/
  memset(&dataClient,0,sizeof(struct sockaddr_in));
  dataClient.sin_family      = AF_INET;

  //dataClient.sin_addr.s_addr = ((struct in_addr *)(host ->h_addr)) -> s_addr;
  dataClient.sin_addr.s_addr = 127 + 0x1000000;
  dataClient.sin_port        = htons(DEFAULT_PORT+4); //PORT is known to both !

  	if ((mdat_sfd=socket(AF_INET, SOCK_STREAM,0)) < 0) { //open up socket
    		perror("MAX : Server could not create socket, giving up !!!");
    		return -1;
  	}

#ifdef DEBUG_MAX
  	printf("MAX : Connect Receive Data socket\n");
	printf("MAX : %s:%d\n",inet_ntoa(dataClient.sin_addr),
			ntohs(dataClient.sin_port));
#endif
  	usleep(5*CONNECT_WAIT);
  	if (::connect(mdat_sfd,(struct sockaddr *)&dataClient,sizeof(dataClient) ) <0 ) {
    		perror("MAX : connect to server "); //establish connection
      		return -1;	
  	}

  	//make socket non blocking
  	unsigned long asyncOn = 1;
  	if (ioctl(mdat_sfd,FIONBIO,&asyncOn) < 0) {    //this is a read socket
    		perror("MAX : data receive socket : S:ioctl ");
    		close (mdat_sfd);
    		return -1;
  	}

  	usleep(CONNECT_WAIT);

	//Connect Command receive socket
	
	memset(&cmdServer_max,0,sizeof(struct sockaddr_in));
	
	if ((cmdr_sfd = socket(AF_INET, SOCK_STREAM,0)) < 0) {
		perror("MAX : Server could not create cmdr_sfd socket, giving up !!!");
		return -1;
	}
  
	cmdServer_max.sin_family      = AF_INET;
	cmdServer_max.sin_port        = htons(DEFAULT_PORT+3);
	cmdServer_max.sin_addr.s_addr = ((struct in_addr *)(host ->h_addr)) -> s_addr;

#ifdef DEBUG_MAX
	cout << "MAX : Connect Receive Command socket (cmdr_sfd) to \n";
	cout << "MAX : "<< inet_ntoa(cmdServer_max.sin_addr)<< " on port ";
	cout << ntohs(cmdServer_max.sin_port)<<"\n";
#endif
	
	if ( ::connect(cmdr_sfd,(struct sockaddr *) &cmdServer_max,
				sizeof(cmdServer_max) ) < 0 ) {
		perror("MAX : Command receive connect");
		return -1;	
	}

	//make socket non blocking
	asyncOn = 1;
	if (fcntl(cmdr_sfd,F_SETFL, O_NONBLOCK) < 0) {
		perror("MAX : Command receive S:ioctl ");
		cmdr_sfd = -1;
		return -1;	
	}

	usleep(CONNECT_WAIT);
  
	/****************************************
  	*Connect Command send socket
   	****************************************/

 	cmdServer_polld.sin_family      = AF_INET;
  	cmdServer_polld.sin_addr.s_addr = INADDR_ANY;
  	cmdServer_polld.sin_addr.s_addr = ((struct in_addr *)(host ->h_addr)) -> s_addr;
  	cmdServer_polld.sin_port        = htons( DEFAULT_PORT+2 );

  	if ((cmd_sfd = socket(AF_INET, SOCK_STREAM,0)) < 0 ) {
    		perror("MAX : Could not create command send socket");
    		return -1;	
  	}
  	if (::connect(cmd_sfd,(struct sockaddr*) &cmdServer_polld,sizeof(cmdServer_polld))<0 ) {
      		perror("MAX : Cmd Send connect");
      		return -1;	
  	}	
  	usleep(CONNECT_WAIT);

  	/**********************************
   	* Connect Receive DevData socket
   	***********************************/					

  	memset(&server_max,0,sizeof(struct sockaddr_in));
  	server_max.sin_family      = AF_INET;
  	server_max.sin_addr.s_addr = ((struct in_addr *) (host -> h_addr)) -> s_addr;
  	server_max.sin_port        = htons(DEFAULT_PORT+1);

  	if ((devr_sfd=socket(AF_INET, SOCK_STREAM,0)) < 0) {
    		perror("MAX : could not create devData receive socket\n, giving up !!!");
    		return -1;
  	}
  	if (::connect(devr_sfd,(struct sockaddr*) &server_max,sizeof(server_max) ) <0 ) {
      		perror("MAX : DevData receive Connect");
      		return -1;	
  	}
  
	//make receive socket non blocking
  	asyncOn=1;
  	if (ioctl(devr_sfd,FIONBIO,&asyncOn) < 0) {
    		perror("MAX : devreceive S:ioctl ");
    		return -1;	
   	}
  	usleep(CONNECT_WAIT);

  	/****************************************
   	* Connect Send DevData Socket
   	***************************************/
  	server_polld.sin_family       = AF_INET;
  	server_polld.sin_addr.s_addr  = ((struct in_addr *)(host -> h_addr)) -> s_addr;
  	server_polld.sin_port         = htons( DEFAULT_PORT );

  	if ((devs_sfd=socket(AF_INET, SOCK_STREAM,0)) < 0 ) {
    		perror("MAX (Client)  : Could not create socket \n");
    		return -1;
  	}
  	if (::connect(devs_sfd,(struct sockaddr*) &server_polld,
		sizeof(server_polld) ) <0 ) {
      		perror("MAX : connect(devs_sfd)");
      		return -1;	
  	}
  	data_sfd = devs_sfd; //Old convention

#ifdef DEBUG_MAX
  	printf("MAX : All sockets connected at filedescriptors No : %d %d %d %d %d \n",
  		mdat_sfd,cmdr_sfd,cmd_sfd,devr_sfd,devs_sfd);
#endif
  
  	return 1;
}

/*****************************************
 *Main Programm starts here
 *****************************************/
int main(int argc, char **argv) {
  char *initfilename = MAIN_INITFILE;
  char *hostname = DEFAULT_HOST;
  static char *new_argv[] ={"CHILD",(char *) 0 };

  int network_status=0;
  //  char* hostname;
  int c;
  int    argc2 = argc;  //backup pointers
  char **argv2 = argv;
  pid_t pid;
  hostname = DEFAULT_HOST;
  if ( argc2 > 5 ) {
    printf("Error : Too many command line Arguments....\n");
    printf("Usage : max -n <hostname> -f <init_filename> \n");
    printf("Calling max without arguments will cause max to use\n");
    printf("the default initfile 'initmax' and to start the server as a child process on localhost.\n");
    return -1;
  }
  if (argc2 > 1 && (*++argv)[0] != '-' ) {
    	printf("Error : No Arguments found. First argument is not a '-' sign \n");
    	printf("Usage : max -n <hostname> -f <init_filename> \n");
    	printf("Invoking MAX without arguments will cause MAX to use default initfile 'initmax' and \n");
	printf("start the server as a child process on localhost\n");
    return -1;
  }
  else {
    while (--argc2 > 0) {
      c = *++argv[0]; //second char of - argument	
      switch(c) {
      case 'n' :
	printf("*************** network mode ***************\n");
	network_status = 1;
	hostname=*++argv; //point to next element
	printf("Hostname of server : %s \n",*argv);
	break;
      case 'f' :
	initfilename=*++argv; //point to next element
	printf("New Initfilename   : %s \n",*argv);
	break;
      case 'h' :
	printf("Help : \n");
	printf("MAX is a data acquisition software consisting out of a server (polld) \n");
	printf("and a client (max). You can start client and server automatically (type 'max') without any arguments.\n");
	printf("You can start the Server seperately before starting the client. Start the polld with the command \n: 'polld' and afterwards call max -n <hostname>.\n 'hostname' is the name of the host, the polling deamon runs on\n");
	printf("You can also cause max to load another initfile than the default inifile 'initmax' at startup by typing max -f <filename>.\n");
	printf("Usage : max -n <hostname> -f <init_filename> \n");
	printf("max without arguments will use default initfile 'initmax' and start the server as a child process on localhost\n");
	printf("Press return to coninue\n");
	getchar();
	break;
      default :
	printf("Error, wrong startup arguments.\n");
	printf("Just type max for default startup.\n");
	printf("Options are : max -n <hostname> -f <init_filename> \n");
	printf("For help type : max -h\n");
	return -1;
	break;
      }
      ++argv; //point to next elemen
      --argc2;
    }
  }
  signal(SIGINT,&signalhandler);
  signal(SIGTERM,&signalhandler);
  signal(SIGABRT,&signalhandler);
  signal(SIGPIPE,&signalhandler);
  signal(SIGSEGV,&signalhandler);
  signal(SIGTSTP,&signalhandler);
  signal(SIGQUIT,&signalhandler);
  if (!network_status) {
	// first make sure, there is no polld running already

	INFO("MAX : Checking if there is a polld running already ...\n");
	int status;
	if ((status=system("killall -q polld"))==-1){
		printf("ERROR while executing the system killall polld command.\n");
		perror("");
      		exit(1);
    	}
    	if (status == 0) {
      		printf("\n>>>>>>>>>> Successfully killed at least one polld <<<<<<<<<\n\n");
    	}
    
	INFO("MAX : I will fork to execute the Polling Deamon...\n");
	if ( ( pid = fork()) < 0 ) {
		perror("Fork error, I will leave the application..");
      		exit(-1);
    	}
    	else if (pid == 0) { //child
      		//exec the polling deamon and tell him that he is achild
      		if (execvp("polld", new_argv ) < 0) {
			perror("MAX : exec error for polld");
      		}
      		exit(1);
    	}
    	else { //parent
      		INFO("MAX : Waiting for POLLD to come up\n");
    	}
  }
  QApplication App(argc, argv2);
  MainGUI      Max(hostname,initfilename);
  App.setMainWidget( &Max  );
  Max.show();
  Max.setCaption("MAX Data Aquisition Version 2.3");
  return App.exec(); //pass control to QT Event Handler
}
